#!/bin/bash -e

BUILD_CHIP=ak3918

usage() {
    echo "./build.sh gsc36xx"
}

check_status()
{
	if [ "$?" != "0" ]; then
		echo "***** Error: build failed!"
		exit 1
	fi
}

pre_dev_gsc36xx() {
    if [ -L include/provision.h ]; then
        rm include/provision.h
    fi
    ln -s $PROV_PATH include/provision.h
    CONFIG_FILE=defconfig_gsc36xx
}

build_dev_uimage() {
    echo "***** Building dev kernel for $BUILD_PRODUCT $BUILD_CHIP"
    echo "Config file: $CONFIG_FILE"
    cp $CONFIG_FILE .config
    make uImage -j$CPU 
	check_status
	echo "***** Build...done"
    cp .config $CONFIG_FILE
    make modules
	#echo "Copying arch/arm/boot/uImage to ./kernels/core.${BUILD_PRODUCT}.${BUILD_CHIP}"
    #cp arch/arm/boot/uImage kernels/core.${BUILD_PRODUCT}.${BUILD_CHIP}
    #cp System.map kernels/System.map.${BUILD_PRODUCT}.${BUILD_CHIP}
}

post_dev_gsc36xx() {
    cp -v arch/arm/boot/uImage ${IMAGE_BUILD}/products/support/${BUILD_PRODUCT}/core.${BUILD_CHIP}
    #cp -v System.map ${IMAGE_BUILD}/products/support/${BUILD_PRODUCT}/System.map.${BUILD_CHIP}
}

#
# Start here.
#
if [ -f 'single_thread' ]; then
    CPU=1
else
    CPU=`cat /proc/cpuinfo | grep -c processor`
fi

BUILD_PRODUCT=$1

if [ "$BUILD_PRODUCT" != "gsc36xx" ]; then
    echo "Invalid product $BUILD_PRODUCT"
    usage
    exit 1
fi

#if [ ! -d kernels ]; then
#    mkdir kernels
#fi

# Note: build_funcs.sh script exports PRODUCT variable. 
#       So we need to use different product variable name in this script to avoid name clash.
. ../../../gsc_environment/build_funcs.sh $BUILD_PRODUCT kernel

# Note: PROV_PATH is exported by $BUILD_PRODUCT_env.sh via build_funcs.sh script.
echo "Provision file: $PROV_PATH"

pre_dev_$BUILD_PRODUCT

build_dev_uimage
check_status

post_dev_$BUILD_PRODUCT
exit 0
