/****************************************************************************
 *
 * FILENAME:            $RCSfile: gs_sec_module.c,v $
 *
 * DESCRIPTION:         Grandstream security module for data encryption/decryption.
 *
 * Copyright (c) 2017 by Grandstream Networks, Inc.
 * All rights reserved.
 *
 ***************************************************************************/

//=======================
//      Includes
//=======================
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <gs/factory.h>
#include <gs/gs_sec_module.h>
#include "linux/crypto.h"


//=======================
//      Defines
//=======================
#define GS_SEC_MODULE_VERSION     "1.0"

/* Use the first 16 bytes of the nvram header as IV */
#define AES_BLOCK_SIZE (32)
#define AES_IV_LEN (16)

//=======================
//     Globals
//=======================



//=======================
//      Locals
//=======================
static DEFINE_MUTEX(gs_sec_mutex);

        /* 
         * Private Functions
         */


        /*
         * Private Data
         */


//=======================
//      Functions
//=======================

//=============================================================================
static int
gs_hw_cipher(
    int enc,
    char * ptr_iv,
    char * data_in,
    char * data_out,
    size_t data_in_len,
    size_t * data_out_len
)
//Description: 
//=============================================================================
{
    return 0; 
}

//=============================================================================
int
gs_sec_nvram_mod_cipher(
    int enc,
    char * data_in,
    size_t data_in_len
)
//Description: 
//=============================================================================
{
    int ret_val = -1;
    char * ptr_data_in = data_in;
    char * ptr_data_out = data_in;
    unsigned int header_offset = 0;
    size_t data_out_len = 0;
    char iv[AES_IV_LEN] = { 0 };

    if ( data_in_len < AES_IV_LEN )
    {
        printk("WARNING!!! Input size too small\n");
        return ret_val;
    }
    
    header_offset = 0; //( NVRAM_HEADER_SIZE + AES_BLOCK_SIZE - 1) / AES_BLOCK_SIZE * AES_BLOCK_SIZE;

    /* If using the NVRAM header, the IV is in the header */
    memcpy( iv, ptr_data_in, AES_IV_LEN );

    /* Before offsetting the ptrs for the nvram header check if we are encrypting in place we need to increment the out ptr as well */
    /* Offset the nvram headr as we are not encrypting it */
    ptr_data_out += header_offset;

    /* Offset the nvram headr size as we are not encrypting it */
    ptr_data_in += header_offset;

    return  gs_hw_cipher( enc, iv, ptr_data_in, ptr_data_out, (data_in_len - header_offset), &data_out_len );
}
EXPORT_SYMBOL( gs_sec_nvram_mod_cipher );
 
//=============================================================================
int
gs_sec_fact_mod_cipher(
    int enc,
    char * data_in,
    size_t data_in_len
)
//Description: 
//=============================================================================
{
    size_t data_out_len = 0; 
    return gs_hw_cipher( enc, NULL, data_in, data_in, data_in_len, &data_out_len );
}
EXPORT_SYMBOL( gs_sec_fact_mod_cipher );


