/**
 * @file camera_tvp5150.c
 * @brief camera driver file
 * Copyright (C) 2011 Anyka (Guangzhou) Microelectronics Technology Co., Ltd
 * @author luweichun
 * @date 2015-11-05
 * @version 1.0
 * @ref
 */ 
#include "ak_sensor_common.h"

#define SENSOR_PWDN_LEVEL		1 
#define SENSOR_RESET_LEVEL		0
#define SENSOR_I2C_ADDR         0xba
#define SENSOR_ID               0x7150
#define SENSOR_MCLK             24
#define SENSOR_REGADDR_BYTE     1
#define SENSOR_DATA_BYTE        1
#define MAX_FPS					25
#define SENSOR_OUTPUT_WIDTH		(tvp5150_res.width)
#define SENSOR_OUTPUT_HEIGHT	(tvp5150_res.height)
#define SENSOR_VALID_OFFSET_X	80
#define SENSOR_VALID_OFFSET_Y	16
#define SENSROR_BUS_TYPE		BUS_TYPE_YUV

#define OV_MAX(a, b)            (((a) < (b) ) ?  (b) : (a))
#define OV_MIN(a, b)            (((a) > (b) ) ?  (b) : (a))
#define OV_CLIP3(low, high, x)  (OV_MAX(OV_MIN((x), high), low))

/***************************V4L2***********************************/
typedef __u64 v4l2_std_id;

/* one bit for each */
#define V4L2_STD_PAL_B          ((v4l2_std_id)0x00000001)
#define V4L2_STD_PAL_B1         ((v4l2_std_id)0x00000002)
#define V4L2_STD_PAL_G          ((v4l2_std_id)0x00000004)
#define V4L2_STD_PAL_H          ((v4l2_std_id)0x00000008)
#define V4L2_STD_PAL_I          ((v4l2_std_id)0x00000010)
#define V4L2_STD_PAL_D          ((v4l2_std_id)0x00000020)
#define V4L2_STD_PAL_D1         ((v4l2_std_id)0x00000040)
#define V4L2_STD_PAL_K          ((v4l2_std_id)0x00000080)

#define V4L2_STD_PAL_M          ((v4l2_std_id)0x00000100)
#define V4L2_STD_PAL_N          ((v4l2_std_id)0x00000200)
#define V4L2_STD_PAL_Nc         ((v4l2_std_id)0x00000400)
#define V4L2_STD_PAL_60         ((v4l2_std_id)0x00000800)

#define V4L2_STD_NTSC_M         ((v4l2_std_id)0x00001000)	/* BTSC */
#define V4L2_STD_NTSC_M_JP      ((v4l2_std_id)0x00002000)	/* EIA-J */
#define V4L2_STD_NTSC_443       ((v4l2_std_id)0x00004000)
#define V4L2_STD_NTSC_M_KR      ((v4l2_std_id)0x00008000)	/* FM A2 */

#define V4L2_STD_SECAM_B        ((v4l2_std_id)0x00010000)
#define V4L2_STD_SECAM_D        ((v4l2_std_id)0x00020000)
#define V4L2_STD_SECAM_G        ((v4l2_std_id)0x00040000)
#define V4L2_STD_SECAM_H        ((v4l2_std_id)0x00080000)
#define V4L2_STD_SECAM_K        ((v4l2_std_id)0x00100000)
#define V4L2_STD_SECAM_K1       ((v4l2_std_id)0x00200000)
#define V4L2_STD_SECAM_L        ((v4l2_std_id)0x00400000)
#define V4L2_STD_SECAM_LC       ((v4l2_std_id)0x00800000)

/* ATSC/HDTV */
#define V4L2_STD_ATSC_8_VSB     ((v4l2_std_id)0x01000000)
#define V4L2_STD_ATSC_16_VSB    ((v4l2_std_id)0x02000000)

/* FIXME:
   Although std_id is 64 bits, there is an issue on PPC32 architecture that
   makes switch(__u64) to break. So, there's a hack on v4l2-common.c rounding
   this value to 32 bits.
   As, currently, the max value is for V4L2_STD_ATSC_16_VSB (30 bits wide),
   it should work fine. However, if needed to add more than two standards,
   v4l2-common.c should be fixed.
 */

/*
 * Some macros to merge video standards in order to make live easier for the
 * drivers and V4L2 applications
 */

/*
 * "Common" NTSC/M - It should be noticed that V4L2_STD_NTSC_443 is
 * Missing here.
 */
#define V4L2_STD_NTSC           (V4L2_STD_NTSC_M	|\
				 V4L2_STD_NTSC_M_JP     |\
				 V4L2_STD_NTSC_M_KR)
/* Secam macros */
#define V4L2_STD_SECAM_DK      	(V4L2_STD_SECAM_D	|\
				 V4L2_STD_SECAM_K	|\
				 V4L2_STD_SECAM_K1)
/* All Secam Standards */
#define V4L2_STD_SECAM		(V4L2_STD_SECAM_B	|\
				 V4L2_STD_SECAM_G	|\
				 V4L2_STD_SECAM_H	|\
				 V4L2_STD_SECAM_DK	|\
				 V4L2_STD_SECAM_L       |\
				 V4L2_STD_SECAM_LC)
/* PAL macros */
#define V4L2_STD_PAL_BG		(V4L2_STD_PAL_B		|\
				 V4L2_STD_PAL_B1	|\
				 V4L2_STD_PAL_G)
#define V4L2_STD_PAL_DK		(V4L2_STD_PAL_D		|\
				 V4L2_STD_PAL_D1	|\
				 V4L2_STD_PAL_K)
/*
 * "Common" PAL - This macro is there to be compatible with the old
 * V4L1 concept of "PAL": /BGDKHI.
 * Several PAL standards are mising here: /M, /N and /Nc
 */
#define V4L2_STD_PAL		(V4L2_STD_PAL_BG	|\
				 V4L2_STD_PAL_DK	|\
				 V4L2_STD_PAL_H		|\
				 V4L2_STD_PAL_I)
/* Chroma "agnostic" standards */
#define V4L2_STD_B		(V4L2_STD_PAL_B		|\
				 V4L2_STD_PAL_B1	|\
				 V4L2_STD_SECAM_B)
#define V4L2_STD_G		(V4L2_STD_PAL_G		|\
				 V4L2_STD_SECAM_G)
#define V4L2_STD_H		(V4L2_STD_PAL_H		|\
				 V4L2_STD_SECAM_H)
#define V4L2_STD_L		(V4L2_STD_SECAM_L	|\
				 V4L2_STD_SECAM_LC)
#define V4L2_STD_GH		(V4L2_STD_G		|\
				 V4L2_STD_H)
#define V4L2_STD_DK		(V4L2_STD_PAL_DK	|\
				 V4L2_STD_SECAM_DK)
#define V4L2_STD_BG		(V4L2_STD_B		|\
				 V4L2_STD_G)
#define V4L2_STD_MN		(V4L2_STD_PAL_M		|\
				 V4L2_STD_PAL_N		|\
				 V4L2_STD_PAL_Nc	|\
				 V4L2_STD_NTSC)

/* Standards where MTS/BTSC stereo could be found */
#define V4L2_STD_MTS		(V4L2_STD_NTSC_M	|\
				 V4L2_STD_PAL_M		|\
				 V4L2_STD_PAL_N		|\
				 V4L2_STD_PAL_Nc)

/* Standards for Countries with 60Hz Line frequency */
#define V4L2_STD_525_60		(V4L2_STD_PAL_M		|\
				 V4L2_STD_PAL_60	|\
				 V4L2_STD_NTSC		|\
				 V4L2_STD_NTSC_443)
/* Standards for Countries with 50Hz Line frequency */
#define V4L2_STD_625_50		(V4L2_STD_PAL		|\
				 V4L2_STD_PAL_N		|\
				 V4L2_STD_PAL_Nc	|\
				 V4L2_STD_SECAM)

#define V4L2_STD_ATSC           (V4L2_STD_ATSC_8_VSB    |\
				 V4L2_STD_ATSC_16_VSB)
/* Macros with none and all analog standards */
#define V4L2_STD_UNKNOWN        0
#define V4L2_STD_ALL            (V4L2_STD_525_60	|\
				 V4L2_STD_625_50)


/* Teletext World System Teletext
   (WST), defined on ITU-R BT.653-2 */
#define V4L2_SLICED_TELETEXT_B          (0x0001)
/* Video Program System, defined on ETS 300 231*/
#define V4L2_SLICED_VPS                 (0x0400)
/* Closed Caption, defined on EIA-608 */
#define V4L2_SLICED_CAPTION_525         (0x1000)
/* Wide Screen System, defined on ITU-R BT1119.1 */
#define V4L2_SLICED_WSS_625             (0x4000)

#define V4L2_SLICED_VBI_525             (V4L2_SLICED_CAPTION_525)
#define V4L2_SLICED_VBI_625             (V4L2_SLICED_TELETEXT_B | V4L2_SLICED_VPS | V4L2_SLICED_WSS_625)
/***************************V4L2***********************************/

struct tvp5150_resolutions {
	int width;
	int height;
};

static int tvp5150_cmos_updata_d_gain(const unsigned int d_gain);
static int tvp5150_cmos_updata_a_gain(const unsigned int a_gain);
static int tvp5150_cmos_updata_exp_time(unsigned int exp_time);
#if 0
static int tvp5150_cmos_update_a_gain_timer(void);
static int tvp5150_reset(void);
static int tvp5150_set_bt(int bt601);
static int tvp5150_log_status(void);
#endif
void tvp5150_test(void);
void tvp5150_change_parm(void);

static int g_fps = MAX_FPS;
static int g_a_gain = 0;
struct tvp5150_resolutions tvp5150_res;

static int tvp5150_sensor_read_register(int reg)
{
	struct ak_sensor_i2c_data i2cdata;

	i2cdata.u8DevAddr           = SENSOR_I2C_ADDR;
	i2cdata.u32RegAddr          = reg;
	i2cdata.u32RegAddrByteNum   = SENSOR_REGADDR_BYTE;
	i2cdata.u32Data             = 0;
	i2cdata.u32DataByteNum      = SENSOR_DATA_BYTE;

	return ak_sensor_read_register(&i2cdata);
}

static int tvp5150_sensor_write_register(int reg, int data)
{
	struct ak_sensor_i2c_data i2cdata;

	i2cdata.u8DevAddr           = SENSOR_I2C_ADDR;
	i2cdata.u32RegAddr          = reg;
	i2cdata.u32RegAddrByteNum   = SENSOR_REGADDR_BYTE;
	i2cdata.u32Data             = data;
	i2cdata.u32DataByteNum      = SENSOR_DATA_BYTE;

	return ak_sensor_write_register(&i2cdata);
}

static int tvp5150_init(const AK_ISP_SENSOR_INIT_PARA *para)
{
	int i;
	AK_ISP_SENSOR_REG_INFO *preg_info;

	preg_info = para->reg_info;
	for (i = 0; i < para->num; i++) {
//value=tvp5150_sensor_read_register(preg_info->reg_addr);
//ak_sensor_print("before w reg:0x%.x, val:0x%.x, to write:0x%.x\n", preg_info->reg_addr, value, preg_info->value);
		tvp5150_sensor_write_register(preg_info->reg_addr, preg_info->value);
//value=tvp5150_sensor_read_register(preg_info->reg_addr);
//ak_sensor_print("after w reg:0x%.x, val:0x%.x\n", preg_info->reg_addr, value);
		preg_info++;
	}

	tvp5150_change_parm();
	return 0;
}

static int tvp5150_read_id(void)
{
    return SENSOR_ID;
}

static int tvp5150_probe_id(void)
{
    u8 value;

    value = tvp5150_sensor_read_register(0x80);
	ak_sensor_print("reg0x80:0x%x\n", value);
	if (value != 0x71)
		goto fail;

    value = tvp5150_sensor_read_register(0x81);
	ak_sensor_print("reg0x81:0x%x\n", value);
	if (value != 0x50)
		goto fail;

#if 0
	tvp5150_test();
	//tvp5150_reset();
	//tvp5150_set_bt(1);
	tvp5150_log_status();
	tvp5150_change_parm();
#endif


    return SENSOR_ID;

fail:
	return 0;
}

static int tvp5150_get_resolution(int *width, int *height)
{
	*width = SENSOR_OUTPUT_WIDTH;
	*height = SENSOR_OUTPUT_HEIGHT;

	return 0;
}

static int tvp5150_get_mclk(void)
{
	return SENSOR_MCLK;
}

static int tvp5150_get_fps(void)
{
	return g_fps;
}

static int tvp5150_get_valid_coordinate(int *x, int *y)
{
	*x = SENSOR_VALID_OFFSET_X;
	*y = SENSOR_VALID_OFFSET_Y;

	return 0;
}

static enum sensor_bus_type tvp5150_get_bus_type(void)
{
	return SENSROR_BUS_TYPE;
}

static int tvp5150_set_power_on(const int pwdn_pin, const int reset_pin)
{  
    ak_sensor_set_pin_as_gpio(pwdn_pin);
    ak_sensor_set_pin_dir(pwdn_pin, 1);
    ak_sensor_set_pin_level(pwdn_pin, !SENSOR_PWDN_LEVEL);    
    ak_sensor_mdelay(10);
    ak_sensor_set_pin_as_gpio(reset_pin);
    ak_sensor_set_pin_dir(reset_pin, 1);
    ak_sensor_set_pin_level(reset_pin, SENSOR_RESET_LEVEL);
    ak_sensor_mdelay(10);
    ak_sensor_set_pin_level(reset_pin, !SENSOR_RESET_LEVEL);
    ak_sensor_mdelay(20);

	g_fps = MAX_FPS;
	ak_sensor_print("%s\n",__func__);
	return 0;
}

static int tvp5150_set_power_off(const int pwdn_pin, const int reset_pin)
{
	//sccb software standby mode
    
    ak_sensor_set_pin_level(pwdn_pin, SENSOR_PWDN_LEVEL);
    ak_sensor_set_pin_dir(reset_pin, 0);

    return 0;
}

static int tvp5150_set_fps(const int fps)
{
	int ret = 0;

	switch (fps) {
	case MAX_FPS:
		break;
	default:
		ak_sensor_print("%s set fps fail\n", __func__);
		ret = -EINVAL;
		break;
	}

	if (!ret)
		g_fps = fps;

	return ret;
}

static int tvp5150_set_standby_in(const int pwdn_pin, const int reset_pin)
{
	return 0;
}

static int tvp5150_set_standby_out(const int pwdn_pin, const int reset_pin)
{
	return 0;
}

static AK_ISP_SENSOR_CB tvp5150_callback = 
{
	.sensor_init_func 				= tvp5150_init,
	.sensor_read_reg_func			= tvp5150_sensor_read_register,
	.sensor_write_reg_func			= tvp5150_sensor_write_register,
	.sensor_read_id_func			= tvp5150_read_id,
	.sensor_update_a_gain_func		= tvp5150_cmos_updata_a_gain,
	.sensor_update_d_gain_func		= tvp5150_cmos_updata_d_gain,
	.sensor_updata_exp_time_func	= tvp5150_cmos_updata_exp_time,
//	.sensor_timer_func				= tvp5150_cmos_update_a_gain_timer,

	.sensor_probe_id_func			= tvp5150_probe_id,	//use IIC bus
	.sensor_get_resolution_func		= tvp5150_get_resolution,
	.sensor_get_mclk_func			= tvp5150_get_mclk,
	.sensor_get_fps_func			= tvp5150_get_fps,
	.sensor_get_valid_coordinate_func	= tvp5150_get_valid_coordinate,
	.sensor_get_bus_type_func		= tvp5150_get_bus_type,

	.sensor_set_power_on_func		= tvp5150_set_power_on,
	.sensor_set_power_off_func		= tvp5150_set_power_off,
	.sensor_set_fps_func			= tvp5150_set_fps,
	.sensor_set_standby_in_func		= tvp5150_set_standby_in,
	.sensor_set_standby_out_func	= tvp5150_set_standby_out
};

AK_SENSOR_MODULE(tvp5150_callback, tvp5150)

///////////////////////////////////////////////////////////////////////////////////////////
static int a_gain_2x_now=1;
static int a_gain_4x_now=1;
static int a_gain_2x_prev=1;
static int a_gain_4x_prev=1;
//static int a_gain_2x_4x_locked=0;
static unsigned int _cmos_gains_convert(unsigned int a_gain, unsigned int d_gain,
		unsigned int *a_gain_out ,unsigned int *d_gain_out)
{
	//ģ??????????ת???????????㷨??????????ת????sensor????????ʽ
	unsigned int ag_interget_table[61] = {0,  1, 2, 2, 4, 4, 4, 4, 8, 8, 8, 8,
        8, 8, 8, 8,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,32,32,32,32,
        32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32};
    unsigned int ag_integer, ag_fraction, tmp;
	unsigned int ag_return;
 	tmp =  a_gain>>4;
	if(tmp == 0)
	{
		tmp = 1;
		ak_sensor_print("Div ERROR 000!\n");
	}
	//ak_sensor_print(" a_gain = %d ",a_gain);
    tmp =  OV_CLIP3(0, 60, tmp);
    ag_integer =  ag_interget_table[tmp];
    ag_fraction = (a_gain*17)/(16*ag_integer)- 17;
    ag_fraction =  OV_CLIP3(0, 15, ag_fraction);
    //ak_sensor_print(" ag_integer = %d ",ag_integer);
	//ak_sensor_print(" ag_fraction = %d\n", ag_fraction);
    
    switch (ag_integer)
    {
        case 1 :
            //ak_sensor_print("case 1\n");
        	//tvp5150_sensor_write_register(0x3610,0x03);
        	//tvp5150_sensor_write_register(0x3605,0x0c);
            ag_integer = 0x00;
			a_gain_2x_now = 1;
			a_gain_4x_now = 1;
            break;
        case 2 :
        	//ak_sensor_print("case 2\n");
        	//tvp5150_sensor_write_register(0x3610,0x03);
        	//tvp5150_sensor_write_register(0x3605,0x0c);
            ag_integer = 0x10;
			a_gain_2x_now = 1;
			a_gain_4x_now = 1;
            break;
        case 4 :
        	//ak_sensor_print("case 4\n");
        	//tvp5150_sensor_write_register(0x3610,0x03);
        	//tvp5150_sensor_write_register(0x3605,0x0c);
            ag_integer = 0x30;
			a_gain_2x_now = 1;
			a_gain_4x_now = 1;
            break;
        case 8 :
        	//ak_sensor_print("case 8\n");
        	//tvp5150_sensor_write_register(0x3610,0x03);
        	//tvp5150_sensor_write_register(0x3605,0x0c);
            ag_integer = 0x70;
			a_gain_2x_now = 1;
			a_gain_4x_now = 1;
            break;
        case 16 :
        	//ak_sensor_print("case 16\n");
        	//tvp5150_sensor_write_register(0x3610,0x1b);
        	//tvp5150_sensor_write_register(0x3605,0x0c);
            ag_integer = 0x70;
			a_gain_2x_now = 2;
			a_gain_4x_now = 1;
            break;
        case 32 :
        	//ak_sensor_print("case 32\n");
       	 	//tvp5150_sensor_write_register(0x3610,0x1b);
       	 	//tvp5150_sensor_write_register(0x3605,0x3c);
            ag_integer = 0x70;
			a_gain_2x_now = 2;
			a_gain_4x_now = 2;
            break;
        default:
        	//ak_sensor_print("case default\n");
        	//tvp5150_sensor_write_register(0x3610,0x03);
        	//tvp5150_sensor_write_register(0x3605,0x0c);
            ag_integer = 0x00;
            break;
    }
    //ak_sensor_print("ag_integer=%x ag_fraction=%x a_gain_2x_now=%x, a_gain_4x_now=%x\n",
	//	  ag_integer,ag_fraction,a_gain_2x_now,a_gain_4x_now);
    ag_return = ag_integer|ag_fraction;
	*a_gain_out = ag_return ; 
 
	return ag_return;
}

static int tvp5150_cmos_updata_d_gain(const unsigned int d_gain)
{
	//?????????????Ļص?????
	return 0;
}

/*
static int tvp5150_updata_2x_4x_a_gain(void)
{
     //a_gain_2x_now = 2;
	// a_gain_4x_now = 2;
	int all_2x_4x_gain=0;
	    a_gain_2x_4x_locked=0;
	 if((a_gain_2x_prev==1)&&(a_gain_2x_now==2))
	 {
		a_gain_2x_4x_locked =AK_TRUE;
		all_2x_4x_gain = a_gain_2x_prev*a_gain_4x_now;
	 }		
	 else if((a_gain_4x_prev==1)&&(a_gain_4x_now==2))
	 {
		a_gain_2x_4x_locked = AK_TRUE;
		all_2x_4x_gain = a_gain_2x_now*a_gain_4x_prev;
	 }
	 else if((a_gain_2x_prev==2)&&(a_gain_2x_now==1))
	 {
		a_gain_2x_4x_locked = AK_TRUE;
		all_2x_4x_gain = a_gain_2x_prev*a_gain_4x_now;
	 }
	 else if((a_gain_4x_prev==2)&&(a_gain_4x_now==1))
	 {
		a_gain_2x_4x_locked = AK_TRUE;
		all_2x_4x_gain = a_gain_2x_now*a_gain_4x_prev;
	 }
	 
	   if(a_gain_2x_4x_locked!=AK_TRUE)
	   {
	     all_2x_4x_gain=a_gain_2x_now*a_gain_4x_now;
	     switch(all_2x_4x_gain)
	     {
		 	case 1:
			tvp5150_sensor_write_register(0x3610,0x03);
        	tvp5150_sensor_write_register(0x3605,0x0c);
		
			  break;
			case 2:
			tvp5150_sensor_write_register(0x3610,0x1b);
        	tvp5150_sensor_write_register(0x3605,0x0c);
			
			  break;
			case 4:
			tvp5150_sensor_write_register(0x3610,0x1b);
       	 	tvp5150_sensor_write_register(0x3605,0x3c);
			
			   break;
			default:
			   break;
	     }
	   
	   }
	   else   //дȥֵһεֵ;
	   {
	     //all_2x_4x_gain=a_gain_2x_now*a_gain_4x_now;
	     switch(all_2x_4x_gain)
	     {
		 	case 1:
			tvp5150_sensor_write_register(0x3610,0x03);
        	tvp5150_sensor_write_register(0x3605,0x0c);
			
			  break;
			case 2:
			tvp5150_sensor_write_register(0x3610,0x1b);
        	tvp5150_sensor_write_register(0x3605,0x0c);
		
			  break;
			case 4:
			tvp5150_sensor_write_register(0x3610,0x1b);
       	 	tvp5150_sensor_write_register(0x3605,0x3c);
			
			   break;
			default:
			   break;
	     }
	      
	   }

	 a_gain_2x_prev = a_gain_2x_now;
	 a_gain_4x_prev = a_gain_4x_now;
	 return 0;	 
}*/
static int tvp5150_updata_2x_4x_a_gain(void)
{
    int   all_2x_4x_gain;
	   {
	     all_2x_4x_gain=a_gain_2x_now*a_gain_4x_now;
	     switch(all_2x_4x_gain)
	     {
		 	case 1:
//			tvp5150_sensor_write_register(0x3610,0x03);
//       	tvp5150_sensor_write_register(0x3605,0x0c);
		
			  break;
			case 2:
//			tvp5150_sensor_write_register(0x3610,0x1b);
//       	tvp5150_sensor_write_register(0x3605,0x0c);
			
			  break;
			case 4:
//			tvp5150_sensor_write_register(0x3610,0x1b);
//       	tvp5150_sensor_write_register(0x3605,0x3c);
			
			   break;
			default:
			   break;
	     }
	   
	   }
	   
	 a_gain_2x_prev = a_gain_2x_now;
	 a_gain_4x_prev = a_gain_4x_now;
	 return 0;	 
}

#define SKIP_FRAME_NUM	2
static int tvp5150_cmos_updata_a_gain(const unsigned int a_gain)
{
	//??????ģ???????Ļص?????
	unsigned short ag_value;
	unsigned int tmp_a_gain;
	unsigned int tmp_d_gain;
	unsigned int tmp_a_gain_out;
	unsigned int tmp_d_gain_out;
	int ret = 0;
	
	g_a_gain  = a_gain;

	tmp_a_gain = a_gain>>4;
	tmp_d_gain = 0;

	//ag_value = cmos_gains_update(isp,isp->aec_param.current_a_gain);
	_cmos_gains_convert(tmp_a_gain, tmp_d_gain,  &tmp_a_gain_out ,&tmp_d_gain_out);
	
	if(a_gain_2x_prev!=a_gain_2x_now || a_gain_4x_now!=a_gain_4x_prev)
		ret = SKIP_FRAME_NUM;
	ag_value = tmp_a_gain_out;
	tvp5150_updata_2x_4x_a_gain();
//	tvp5150_sensor_write_register(0x3e09, Again_limit(tmp_a_gain_out));
	//ak_sensor_print(" 0x3e09 = %d\n", tvp5150_sensor_read_register(0x3e09));
	//ak_sensor_print(" 0x3610 = %d\n", tvp5150_sensor_read_register(0x3610));
	//ak_sensor_print(" 0x3605 = %d\n", tvp5150_sensor_read_register(0x3605));
	//ak_sensor_print(" tmp_a_gain = %d ",tmp_a_gain);
	//ak_sensor_print(" tmp_a_gain_out = %d\n", tmp_a_gain_out);
	//updata_auto_exposure_num++;
	//isp->aec_param.current_exposure_time = tmp_exposure_time;
	return ret;
}


static int tvp5150_cmos_updata_exp_time(unsigned int exp_time)
{
    //?????ع?ʱ???Ļص?????
    unsigned char exposure_time_msb;
	unsigned char exposure_time_lsb;
	unsigned char tem=0;
 	//ak_sensor_print("exp_time=%d ",exp_time);
    exposure_time_msb =(exp_time>>4)&0xff;
    exposure_time_lsb =((exp_time)&0xf)<<4;
//   tem=tvp5150_sensor_read_register(0x3e02)&0x0f;
    exposure_time_lsb =exposure_time_lsb|tem;
	//ak_sensor_print("msb = %d  ",exposure_time_msb);
	//ak_sensor_print("lsb = %d\n",exposure_time_lsb);
//	tvp5150_sensor_write_register(0x3e01,exposure_time_msb);
//	tvp5150_sensor_write_register(0x3e02,exposure_time_lsb);
    return 0;
}

#if 0
static int tvp5150_cmos_update_a_gain_timer(void)
{
	#define UPDATE_A_GAIN_TIMER_PERIOD	(5)	//s
	static struct timeval stv = {.tv_sec = 0,};
	struct timeval tv;

	do_gettimeofday(&tv);

	if (tv.tv_sec >= stv.tv_sec + UPDATE_A_GAIN_TIMER_PERIOD) {
		stv.tv_sec = tv.tv_sec;
		//tvp5150_cmos_updata_a_gain(g_a_gain);
	}

	return 0;
}
#endif

/********************************************************************************/
/********************************************************************************/
/********************************************************************************/
/********************************************************************************/
/********************************************************************************/

/************************PETER ADD FOR TEST*************************/
/*
 * tvp5150 - Texas Instruments TVP5150A/AM1 video decoder registers
 *
 * Copyright (c) 2005,2006 Mauro Carvalho Chehab (mchehab@infradead.org)
 * This code is placed under the terms of the GNU General Public License v2
 */

#define TVP5150_VD_IN_SRC_SEL_1      0x00 /* Video input source selection #1 */
#define TVP5150_ANAL_CHL_CTL         0x01 /* Analog channel controls */
#define TVP5150_OP_MODE_CTL          0x02 /* Operation mode controls */
#define TVP5150_MISC_CTL             0x03 /* Miscellaneous controls */
#define TVP5150_AUTOSW_MSK           0x04 /* Autoswitch mask: TVP5150A / TVP5150AM */

/* Reserved 05h */

#define TVP5150_COLOR_KIL_THSH_CTL   0x06 /* Color killer threshold control */
#define TVP5150_LUMA_PROC_CTL_1      0x07 /* Luminance processing control #1 */
#define TVP5150_LUMA_PROC_CTL_2      0x08 /* Luminance processing control #2 */
#define TVP5150_BRIGHT_CTL           0x09 /* Brightness control */
#define TVP5150_SATURATION_CTL       0x0a /* Color saturation control */
#define TVP5150_HUE_CTL              0x0b /* Hue control */
#define TVP5150_CONTRAST_CTL         0x0c /* Contrast control */
#define TVP5150_DATA_RATE_SEL        0x0d /* Outputs and data rates select */
#define TVP5150_LUMA_PROC_CTL_3      0x0e /* Luminance processing control #3 */
#define TVP5150_CONF_SHARED_PIN      0x0f /* Configuration shared pins */

/* Reserved 10h */

#define TVP5150_ACT_VD_CROP_ST_MSB   0x11 /* Active video cropping start MSB */
#define TVP5150_ACT_VD_CROP_ST_LSB   0x12 /* Active video cropping start LSB */
#define TVP5150_ACT_VD_CROP_STP_MSB  0x13 /* Active video cropping stop MSB */
#define TVP5150_ACT_VD_CROP_STP_LSB  0x14 /* Active video cropping stop LSB */
#define TVP5150_GENLOCK              0x15 /* Genlock/RTC */
#define TVP5150_HORIZ_SYNC_START     0x16 /* Horizontal sync start */

/* Reserved 17h */

#define TVP5150_VERT_BLANKING_START 0x18 /* Vertical blanking start */
#define TVP5150_VERT_BLANKING_STOP  0x19 /* Vertical blanking stop */
#define TVP5150_CHROMA_PROC_CTL_1   0x1a /* Chrominance processing control #1 */
#define TVP5150_CHROMA_PROC_CTL_2   0x1b /* Chrominance processing control #2 */
#define TVP5150_INT_RESET_REG_B     0x1c /* Interrupt reset register B */
#define TVP5150_INT_ENABLE_REG_B    0x1d /* Interrupt enable register B */
#define TVP5150_INTT_CONFIG_REG_B   0x1e /* Interrupt configuration register B */

/* Reserved 1Fh-27h */

#define VIDEO_STD_MASK			 (0x07 >> 1)
#define TVP5150_VIDEO_STD                0x28 /* Video standard */
#define VIDEO_STD_AUTO_SWITCH_BIT	 0x00
#define VIDEO_STD_NTSC_MJ_BIT		 0x02
#define VIDEO_STD_PAL_BDGHIN_BIT	 0x04
#define VIDEO_STD_PAL_M_BIT		 0x06
#define VIDEO_STD_PAL_COMBINATION_N_BIT	 0x08
#define VIDEO_STD_NTSC_4_43_BIT		 0x0a
#define VIDEO_STD_SECAM_BIT		 0x0c

#define VIDEO_STD_NTSC_MJ_BIT_AS                 0x01
#define VIDEO_STD_PAL_BDGHIN_BIT_AS              0x03
#define VIDEO_STD_PAL_M_BIT_AS		         0x05
#define VIDEO_STD_PAL_COMBINATION_N_BIT_AS	 0x07
#define VIDEO_STD_NTSC_4_43_BIT_AS		 0x09
#define VIDEO_STD_SECAM_BIT_AS		         0x0b

/* Reserved 29h-2bh */

#define TVP5150_CB_GAIN_FACT        0x2c /* Cb gain factor */
#define TVP5150_CR_GAIN_FACTOR      0x2d /* Cr gain factor */
#define TVP5150_MACROVISION_ON_CTR  0x2e /* Macrovision on counter */
#define TVP5150_MACROVISION_OFF_CTR 0x2f /* Macrovision off counter */
#define TVP5150_REV_SELECT          0x30 /* revision select (TVP5150AM1 only) */

/* Reserved	31h-7Fh */

#define TVP5150_MSB_DEV_ID          0x80 /* MSB of device ID */
#define TVP5150_LSB_DEV_ID          0x81 /* LSB of device ID */
#define TVP5150_ROM_MAJOR_VER       0x82 /* ROM major version */
#define TVP5150_ROM_MINOR_VER       0x83 /* ROM minor version */
#define TVP5150_VERT_LN_COUNT_MSB   0x84 /* Vertical line count MSB */
#define TVP5150_VERT_LN_COUNT_LSB   0x85 /* Vertical line count LSB */
#define TVP5150_INT_STATUS_REG_B    0x86 /* Interrupt status register B */
#define TVP5150_INT_ACTIVE_REG_B    0x87 /* Interrupt active register B */
#define TVP5150_STATUS_REG_1        0x88 /* Status register #1 */
#define TVP5150_STATUS_REG_2        0x89 /* Status register #2 */
#define TVP5150_STATUS_REG_3        0x8a /* Status register #3 */
#define TVP5150_STATUS_REG_4        0x8b /* Status register #4 */
#define TVP5150_STATUS_REG_5        0x8c /* Status register #5 */
/* Reserved	8Dh-8Fh */
 /* Closed caption data registers */
#define TVP5150_CC_DATA_INI         0x90
#define TVP5150_CC_DATA_END         0x93

 /* WSS data registers */
#define TVP5150_WSS_DATA_INI        0x94
#define TVP5150_WSS_DATA_END        0x99

/* VPS data registers */
#define TVP5150_VPS_DATA_INI        0x9a
#define TVP5150_VPS_DATA_END        0xa6

/* VITC data registers */
#define TVP5150_VITC_DATA_INI       0xa7
#define TVP5150_VITC_DATA_END       0xaf

#define TVP5150_VBI_FIFO_READ_DATA  0xb0 /* VBI FIFO read data */

/* Teletext filter 1 */
#define TVP5150_TELETEXT_FIL1_INI  0xb1
#define TVP5150_TELETEXT_FIL1_END  0xb5

/* Teletext filter 2 */
#define TVP5150_TELETEXT_FIL2_INI  0xb6
#define TVP5150_TELETEXT_FIL2_END  0xba

#define TVP5150_TELETEXT_FIL_ENA    0xbb /* Teletext filter enable */
/* Reserved	BCh-BFh */
#define TVP5150_INT_STATUS_REG_A    0xc0 /* Interrupt status register A */
#define TVP5150_INT_ENABLE_REG_A    0xc1 /* Interrupt enable register A */
#define TVP5150_INT_CONF            0xc2 /* Interrupt configuration */
#define TVP5150_VDP_CONF_RAM_DATA   0xc3 /* VDP configuration RAM data */
#define TVP5150_CONF_RAM_ADDR_LOW   0xc4 /* Configuration RAM address low byte */
#define TVP5150_CONF_RAM_ADDR_HIGH  0xc5 /* Configuration RAM address high byte */
#define TVP5150_VDP_STATUS_REG      0xc6 /* VDP status register */
#define TVP5150_FIFO_WORD_COUNT     0xc7 /* FIFO word count */
#define TVP5150_FIFO_INT_THRESHOLD  0xc8 /* FIFO interrupt threshold */
#define TVP5150_FIFO_RESET          0xc9 /* FIFO reset */
#define TVP5150_LINE_NUMBER_INT     0xca /* Line number interrupt */
#define TVP5150_PIX_ALIGN_REG_LOW   0xcb /* Pixel alignment register low byte */
#define TVP5150_PIX_ALIGN_REG_HIGH  0xcc /* Pixel alignment register high byte */
#define TVP5150_FIFO_OUT_CTRL       0xcd /* FIFO output control */
/* Reserved	CEh */
#define TVP5150_FULL_FIELD_ENA      0xcf /* Full field enable 1 */

/* Line mode registers */
#define TVP5150_LINE_MODE_INI       0xd0
#define TVP5150_LINE_MODE_END       0xfb

#define TVP5150_FULL_FIELD_MODE_REG 0xfc /* Full field mode register */
/* Reserved	FDh-FFh */

/*
 * tvp5150 - Texas Instruments TVP5150A/AM1 video decoder driver
 *
 * Copyright (c) 2005,2006 Mauro Carvalho Chehab (mchehab@infradead.org)
 * This code is placed under the terms of the GNU General Public License v2
 */

#define TVP5150_H_MAX		720
#define TVP5150_V_MAX_525_60	480
#define TVP5150_V_MAX_OTHERS	576
#define TVP5150_MAX_CROP_LEFT	511
#define TVP5150_MAX_CROP_TOP	127
#define TVP5150_CROP_SHIFT	2

/* TVP5150 HW inputs */
#define TVP5150_COMPOSITE0 0
#define TVP5150_COMPOSITE1 1
#define TVP5150_SVIDEO     2

/* TVP5150 HW outputs */
#define TVP5150_NORMAL       0
#define TVP5150_BLACK_SCREEN 1

struct i2c_reg_value {
	unsigned char reg;
	unsigned char value;
};

struct tvp5150 {
//	struct v4l2_subdev sd;
//	struct v4l2_ctrl_handler hdl;
//	struct v4l2_rect rect;

	v4l2_std_id norm;	/* Current set standard */
	u32 input;
	u32 output;
	int enable;
};

struct tvp5150_vbi_type {
	unsigned int vbi_type;
	unsigned int ini_line;
	unsigned int end_line;
	unsigned int by_field :1;
};

struct i2c_vbi_ram_value {
	u16 reg;
	struct tvp5150_vbi_type type;
	unsigned char values[16];
};

/* Default values as sugested at TVP5150AM1 datasheet */
static const struct i2c_reg_value tvp5150_init_default[] = {
	{ /* 0x00 */
		TVP5150_VD_IN_SRC_SEL_1,0x00
	},
	{ /* 0x01 */
		TVP5150_ANAL_CHL_CTL,0x15
	},
	{ /* 0x02 */
		TVP5150_OP_MODE_CTL,0x00
	},
	{ /* 0x03 */
		TVP5150_MISC_CTL,0x01
	},
	{ /* 0x06 */
		TVP5150_COLOR_KIL_THSH_CTL,0x10
	},
	{ /* 0x07 */
		TVP5150_LUMA_PROC_CTL_1,0x60
	},
	{ /* 0x08 */
		TVP5150_LUMA_PROC_CTL_2,0x00
	},
	{ /* 0x09 */
		TVP5150_BRIGHT_CTL,0x80
	},
	{ /* 0x0a */
		TVP5150_SATURATION_CTL,0x80
	},
	{ /* 0x0b */
		TVP5150_HUE_CTL,0x00
	},
	{ /* 0x0c */
		TVP5150_CONTRAST_CTL,0x80
	},
	{ /* 0x0d */
		TVP5150_DATA_RATE_SEL,0x47
	},
	{ /* 0x0e */
		TVP5150_LUMA_PROC_CTL_3,0x00
	},
	{ /* 0x0f */
		TVP5150_CONF_SHARED_PIN,0x08
	},
	{ /* 0x11 */
		TVP5150_ACT_VD_CROP_ST_MSB,0x00
	},
	{ /* 0x12 */
		TVP5150_ACT_VD_CROP_ST_LSB,0x00
	},
	{ /* 0x13 */
		TVP5150_ACT_VD_CROP_STP_MSB,0x00
	},
	{ /* 0x14 */
		TVP5150_ACT_VD_CROP_STP_LSB,0x00
	},
	{ /* 0x15 */
		TVP5150_GENLOCK,0x01
	},
	{ /* 0x16 */
		TVP5150_HORIZ_SYNC_START,0x80
	},
	{ /* 0x18 */
		TVP5150_VERT_BLANKING_START,0x00
	},
	{ /* 0x19 */
		TVP5150_VERT_BLANKING_STOP,0x00
	},
	{ /* 0x1a */
		TVP5150_CHROMA_PROC_CTL_1,0x0c
	},
	{ /* 0x1b */
		TVP5150_CHROMA_PROC_CTL_2,0x14
	},
	{ /* 0x1c */
		TVP5150_INT_RESET_REG_B,0x00
	},
	{ /* 0x1d */
		TVP5150_INT_ENABLE_REG_B,0x00
	},
	{ /* 0x1e */
		TVP5150_INTT_CONFIG_REG_B,0x00
	},
	{ /* 0x28 */
		TVP5150_VIDEO_STD,0x00
	},
	{ /* 0x2e */
		TVP5150_MACROVISION_ON_CTR,0x0f
	},
	{ /* 0x2f */
		TVP5150_MACROVISION_OFF_CTR,0x01
	},
	{ /* 0xbb */
		TVP5150_TELETEXT_FIL_ENA,0x00
	},
	{ /* 0xc0 */
		TVP5150_INT_STATUS_REG_A,0x00
	},
	{ /* 0xc1 */
		TVP5150_INT_ENABLE_REG_A,0x00
	},
	{ /* 0xc2 */
		TVP5150_INT_CONF,0x04
	},
	{ /* 0xc8 */
		TVP5150_FIFO_INT_THRESHOLD,0x80
	},
	{ /* 0xc9 */
		TVP5150_FIFO_RESET,0x00
	},
	{ /* 0xca */
		TVP5150_LINE_NUMBER_INT,0x00
	},
	{ /* 0xcb */
		TVP5150_PIX_ALIGN_REG_LOW,0x4e
	},
	{ /* 0xcc */
		TVP5150_PIX_ALIGN_REG_HIGH,0x00
	},
	{ /* 0xcd */
		TVP5150_FIFO_OUT_CTRL,0x01
	},
	{ /* 0xcf */
		TVP5150_FULL_FIELD_ENA,0x00
	},
	{ /* 0xd0 */
		TVP5150_LINE_MODE_INI,0x00
	},
	{ /* 0xfc */
		TVP5150_FULL_FIELD_MODE_REG,0x7f
	},
	{ /* end of data */
		0xff,0xff
	}
};

/* This struct have the values for each supported VBI Standard
 * by
 tvp5150_vbi_types should follow the same order as vbi_ram_default
 * value 0 means rom position 0x10, value 1 means rom position 0x30
 * and so on. There are 16 possible locations from 0 to 15.
 */
#if 0
static struct i2c_vbi_ram_value vbi_ram_default[] =
{
	/* FIXME: Current api doesn't handle all VBI types, those not
	   yet supported are placed under #if 0 */
#if 0
	{0x010, /* Teletext, SECAM, WST System A */
		{V4L2_SLICED_TELETEXT_SECAM,6,23,1},
		{ 0xaa, 0xaa, 0xff, 0xff, 0xe7, 0x2e, 0x20, 0x26,
		  0xe6, 0xb4, 0x0e, 0x00, 0x00, 0x00, 0x10, 0x00 }
	},
#endif
	{0x030, /* Teletext, PAL, WST System B */
		{V4L2_SLICED_TELETEXT_B,6,22,1},
		{ 0xaa, 0xaa, 0xff, 0xff, 0x27, 0x2e, 0x20, 0x2b,
		  0xa6, 0x72, 0x10, 0x00, 0x00, 0x00, 0x10, 0x00 }
	},
#if 0
	{0x050, /* Teletext, PAL, WST System C */
		{V4L2_SLICED_TELETEXT_PAL_C,6,22,1},
		{ 0xaa, 0xaa, 0xff, 0xff, 0xe7, 0x2e, 0x20, 0x22,
		  0xa6, 0x98, 0x0d, 0x00, 0x00, 0x00, 0x10, 0x00 }
	},
	{0x070, /* Teletext, NTSC, WST System B */
		{V4L2_SLICED_TELETEXT_NTSC_B,10,21,1},
		{ 0xaa, 0xaa, 0xff, 0xff, 0x27, 0x2e, 0x20, 0x23,
		  0x69, 0x93, 0x0d, 0x00, 0x00, 0x00, 0x10, 0x00 }
	},
	{0x090, /* Tetetext, NTSC NABTS System C */
		{V4L2_SLICED_TELETEXT_NTSC_C,10,21,1},
		{ 0xaa, 0xaa, 0xff, 0xff, 0xe7, 0x2e, 0x20, 0x22,
		  0x69, 0x93, 0x0d, 0x00, 0x00, 0x00, 0x15, 0x00 }
	},
	{0x0b0, /* Teletext, NTSC-J, NABTS System D */
		{V4L2_SLICED_TELETEXT_NTSC_D,10,21,1},
		{ 0xaa, 0xaa, 0xff, 0xff, 0xa7, 0x2e, 0x20, 0x23,
		  0x69, 0x93, 0x0d, 0x00, 0x00, 0x00, 0x10, 0x00 }
	},
	{0x0d0, /* Closed Caption, PAL/SECAM */
		{V4L2_SLICED_CAPTION_625,22,22,1},
		{ 0xaa, 0x2a, 0xff, 0x3f, 0x04, 0x51, 0x6e, 0x02,
		  0xa6, 0x7b, 0x09, 0x00, 0x00, 0x00, 0x27, 0x00 }
	},
#endif
	{0x0f0, /* Closed Caption, NTSC */
		{V4L2_SLICED_CAPTION_525,21,21,1},
		{ 0xaa, 0x2a, 0xff, 0x3f, 0x04, 0x51, 0x6e, 0x02,
		  0x69, 0x8c, 0x09, 0x00, 0x00, 0x00, 0x27, 0x00 }
	},
	{0x110, /* Wide Screen Signal, PAL/SECAM */
		{V4L2_SLICED_WSS_625,23,23,1},
		{ 0x5b, 0x55, 0xc5, 0xff, 0x00, 0x71, 0x6e, 0x42,
		  0xa6, 0xcd, 0x0f, 0x00, 0x00, 0x00, 0x3a, 0x00 }
	},
#if 0
	{0x130, /* Wide Screen Signal, NTSC C */
		{V4L2_SLICED_WSS_525,20,20,1},
		{ 0x38, 0x00, 0x3f, 0x00, 0x00, 0x71, 0x6e, 0x43,
		  0x69, 0x7c, 0x08, 0x00, 0x00, 0x00, 0x39, 0x00 }
	},
	{0x150, /* Vertical Interval Timecode (VITC), PAL/SECAM */
		{V4l2_SLICED_VITC_625,6,22,0},
		{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x8f, 0x6d, 0x49,
		  0xa6, 0x85, 0x08, 0x00, 0x00, 0x00, 0x4c, 0x00 }
	},
	{0x170, /* Vertical Interval Timecode (VITC), NTSC */
		{V4l2_SLICED_VITC_525,10,20,0},
		{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x8f, 0x6d, 0x49,
		  0x69, 0x94, 0x08, 0x00, 0x00, 0x00, 0x4c, 0x00 }
	},
#endif
	{0x190, /* Video Program System (VPS), PAL */
		{V4L2_SLICED_VPS,16,16,0},
		{ 0xaa, 0xaa, 0xff, 0xff, 0xba, 0xce, 0x2b, 0x0d,
		  0xa6, 0xda, 0x0b, 0x00, 0x00, 0x00, 0x60, 0x00 }
	},
	/* 0x1d0 User programmable */

	/* End of struct */
	{ (u16)-1 }
};
#endif

/* Default values as sugested at TVP5150AM1 datasheet */
static const struct i2c_reg_value tvp5150_init_enable[] = {
	{
		TVP5150_CONF_SHARED_PIN, 2
	},{	/* Automatic offset and AGC enabled */
		TVP5150_ANAL_CHL_CTL, 0x15
	},{	/* Activate YCrCb output 0x9 or 0xd ? */
		TVP5150_MISC_CTL, 0x6f
	},{	/* Activates video std autodetection for all standards */
		TVP5150_AUTOSW_MSK, 0x0
	},{	/* Default format: 0x47. For 4:2:2: 0x40 */
		TVP5150_DATA_RATE_SEL, 0x47
	},{
		TVP5150_CHROMA_PROC_CTL_1, 0x0c
	},{
		TVP5150_CHROMA_PROC_CTL_2, 0x54
	},{	/* Non documented, but initialized on WinTV USB2 */
		0x27, 0x20
	},{
		0xff,0xff
	}
};

#if 0
static int tvp5150_write_inittab(const struct i2c_reg_value *regs)
{
	while (regs->reg != 0xff) {
		tvp5150_sensor_write_register(regs->reg, regs->value);
		regs++;
	}
	return 0;
}

static int tvp5150_vdp_init(const struct i2c_vbi_ram_value *regs)
{
	unsigned int i;

	/* Disable Full Field */
	tvp5150_sensor_write_register(TVP5150_FULL_FIELD_ENA, 0);

	/* Before programming, Line mode should be at 0xff */
	for (i = TVP5150_LINE_MODE_INI; i <= TVP5150_LINE_MODE_END; i++)
		tvp5150_sensor_write_register(i, 0xff);
		//tvp5150_write(sd, i, 0xff);

	/* Load Ram Table */
	while (regs->reg != (u16)-1) {
		tvp5150_sensor_write_register(TVP5150_CONF_RAM_ADDR_HIGH, regs->reg >> 8);
		tvp5150_sensor_write_register(TVP5150_CONF_RAM_ADDR_LOW, regs->reg);
		//tvp5150_write(sd, TVP5150_CONF_RAM_ADDR_HIGH, regs->reg >> 8);
		//tvp5150_write(sd, TVP5150_CONF_RAM_ADDR_LOW, regs->reg);

		for (i = 0; i < 16; i++)
			tvp5150_sensor_write_register(TVP5150_VDP_CONF_RAM_DATA, regs->values[i]);
			//tvp5150_write(sd, TVP5150_VDP_CONF_RAM_DATA, regs->values[i]);

		regs++;
	}
	return 0;
}
#endif

static inline void tvp5150_selmux(struct tvp5150 *decoder)
{
	int opmode = 0;
	int input = 0;
	unsigned char val;

	if ((decoder->output & TVP5150_BLACK_SCREEN) || !decoder->enable)
		input = 8;

	switch (decoder->input) {
	case TVP5150_COMPOSITE1:
		input |= 2;
		/* fall through */
	case TVP5150_COMPOSITE0:
		break;
	case TVP5150_SVIDEO:
	default:
		input |= 1;
		break;
	}

	
	tvp5150_sensor_write_register(TVP5150_OP_MODE_CTL, opmode);
	tvp5150_sensor_write_register(TVP5150_VD_IN_SRC_SEL_1, input);
	//tvp5150_write(sd, TVP5150_OP_MODE_CTL, opmode);
	//tvp5150_write(sd, TVP5150_VD_IN_SRC_SEL_1, input);

	/* Svideo should enable YCrCb output and disable GPCL output
	 * For Composite and TV, it should be the reverse
	 */
	val = tvp5150_sensor_read_register(TVP5150_MISC_CTL);
	//val = tvp5150_sensor_read_register(TVP5150_MISC_CTL);
	if (decoder->input == TVP5150_SVIDEO)
		val = (val & ~0x40) | 0x10;
	else
		val = (val & ~0x10) | 0x40;
	tvp5150_sensor_write_register(TVP5150_MISC_CTL, val);
	//tvp5150_write(sd, TVP5150_MISC_CTL, val);
};

#if 0
static int tvp5150_set_std(v4l2_std_id std)
{
	int fmt = 0;

	/* First tests should be against specific std */

	if (std == V4L2_STD_ALL) {
		fmt = VIDEO_STD_AUTO_SWITCH_BIT;	/* Autodetect mode */
	} else if (std & V4L2_STD_NTSC_443) {
		fmt = VIDEO_STD_NTSC_4_43_BIT;
	} else if (std & V4L2_STD_PAL_M) {
		fmt = VIDEO_STD_PAL_M_BIT;
	} else if (std & (V4L2_STD_PAL_N | V4L2_STD_PAL_Nc)) {
		fmt = VIDEO_STD_PAL_COMBINATION_N_BIT;
	} else {
		/* Then, test against generic ones */
		if (std & V4L2_STD_NTSC)
			fmt = VIDEO_STD_NTSC_MJ_BIT;
		else if (std & V4L2_STD_PAL)
			fmt = VIDEO_STD_PAL_BDGHIN_BIT;
		else if (std & V4L2_STD_SECAM)
			fmt = VIDEO_STD_SECAM_BIT;
	}

	
	tvp5150_sensor_write_register(TVP5150_VIDEO_STD, fmt);
	//tvp5150_write(sd, TVP5150_VIDEO_STD, fmt);
	return 0;
}

static int tvp5150_reset(void)
{
	struct tvp5150 decoder = {
		.norm	= V4L2_STD_ALL,
		.input	= TVP5150_COMPOSITE0,
		.output	= TVP5150_NORMAL,
		.enable = 1,
	};

	/* Initializes TVP5150 to its default values */
	tvp5150_write_inittab(tvp5150_init_default);

	/* Initializes VDP registers */
	tvp5150_vdp_init(vbi_ram_default);

	/* Selects decoder input */
	tvp5150_selmux(&decoder);

	/* Initializes TVP5150 to stream enabled values */
	tvp5150_write_inittab(tvp5150_init_enable);

	/* Initialize image preferences */
	//v4l2_ctrl_handler_setup(&decoder->hdl);

	tvp5150_set_std(decoder.norm);
	return 0;
};

static int tvp5150_set_bt(int bt601)
{
	int tmp;

	if (bt601) {//BT601
		//tmp = 1;//2;
		//sccb_write_data(i2c_addr, TVP5150_CONF_SHARED_PIN, &tmp, 1);
		tmp = 0x15;
		tvp5150_sensor_write_register(TVP5150_ANAL_CHL_CTL, tmp);
		//sccb_write_data(i2c_addr, TVP5150_ANAL_CHL_CTL, &tmp, 1);


		//    tmp = 0x0;
		//    sccb_write_data(i2c_addr, TVP5150_AUTOSW_MSK, &tmp, 1);
		tmp = 0x40;
		tvp5150_sensor_write_register(TVP5150_DATA_RATE_SEL, tmp);
		//sccb_write_data(i2c_addr, TVP5150_DATA_RATE_SEL, &tmp, 1);

		tmp = 0x7f;
		tvp5150_sensor_write_register(TVP5150_HORIZ_SYNC_START, tmp);
		//sccb_write_data(i2c_addr, TVP5150_HORIZ_SYNC_START, &tmp, 1);

		//    tmp = 0x8;
		//    sccb_write_data(i2c_addr, TVP5150_VD_IN_SRC_SEL_1, &tmp, 1);

		//    tmp = 0x0c;
		//    sccb_write_data(i2c_addr, TVP5150_CHROMA_PROC_CTL_1, &tmp, 1);
		//    
		//    tmp = 0x54;
		//    sccb_write_data(i2c_addr, TVP5150_CHROMA_PROC_CTL_2, &tmp, 1);
		//    tmp = 0x20;
		//    sccb_write_data(i2c_addr, 0x27, &tmp, 1);

		tmp = 0x6d;
		tvp5150_sensor_write_register(TVP5150_MISC_CTL, tmp);
		//sccb_write_data(i2c_addr, TVP5150_MISC_CTL, &tmp, 1);
		/*****************end peter add for test*****************/
	} else {
		//BT656
		tmp = 0x69;
		tvp5150_sensor_write_register(TVP5150_MISC_CTL, tmp);
		//sccb_write_data(i2c_addr, TVP5150_MISC_CTL, &tmp, 1);
	}

	return 0;
}

static void dump_reg_range(char *s, u8 init,
				const u8 end, int max_line)
{
	int i = 0;

	while (init != (u8)(end + 1)) {
		if ((i % max_line) == 0) {
			if (i > 0)
				ak_sensor_print("\n");
			ak_sensor_print("tvp5150: %s reg 0x%02x = ", s, init);
		}
		ak_sensor_print("%02x ", tvp5150_sensor_read_register(init));

		init++;
		i++;
	}
	ak_sensor_print("\n");
}
#endif

#if 0
static int tvp5150_log_status(void)
{
	ak_sensor_print("tvp5150: Video input source selection #1 = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_VD_IN_SRC_SEL_1));
	ak_sensor_print("tvp5150: Analog channel controls = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_ANAL_CHL_CTL));
	ak_sensor_print("tvp5150: Operation mode controls = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_OP_MODE_CTL));
	ak_sensor_print("tvp5150: Miscellaneous controls = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_MISC_CTL));
	ak_sensor_print("tvp5150: Autoswitch mask= 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_AUTOSW_MSK));
	ak_sensor_print("tvp5150: Color killer threshold control = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_COLOR_KIL_THSH_CTL));
	ak_sensor_print("tvp5150: Luminance processing controls #1 #2 and #3 = %02x %02x %02x\n",
			tvp5150_sensor_read_register(TVP5150_LUMA_PROC_CTL_1),
			tvp5150_sensor_read_register(TVP5150_LUMA_PROC_CTL_2),
			tvp5150_sensor_read_register(TVP5150_LUMA_PROC_CTL_3));
	ak_sensor_print("tvp5150: Brightness control = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_BRIGHT_CTL));
	ak_sensor_print("tvp5150: Color saturation control = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_SATURATION_CTL));
	ak_sensor_print("tvp5150: Hue control = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_HUE_CTL));
	ak_sensor_print("tvp5150: Contrast control = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_CONTRAST_CTL));
	ak_sensor_print("tvp5150: Outputs and data rates select = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_DATA_RATE_SEL));
	ak_sensor_print("tvp5150: Configuration shared pins = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_CONF_SHARED_PIN));
	ak_sensor_print("tvp5150: Active video cropping start = 0x%02x%02x\n",
			tvp5150_sensor_read_register(TVP5150_ACT_VD_CROP_ST_MSB),
			tvp5150_sensor_read_register(TVP5150_ACT_VD_CROP_ST_LSB));
	ak_sensor_print("tvp5150: Active video cropping stop  = 0x%02x%02x\n",
			tvp5150_sensor_read_register(TVP5150_ACT_VD_CROP_STP_MSB),
			tvp5150_sensor_read_register(TVP5150_ACT_VD_CROP_STP_LSB));
	ak_sensor_print("tvp5150: Genlock/RTC = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_GENLOCK));
	ak_sensor_print("tvp5150: Horizontal sync start = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_HORIZ_SYNC_START));
	ak_sensor_print("tvp5150: Vertical blanking start = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_VERT_BLANKING_START));
	ak_sensor_print("tvp5150: Vertical blanking stop = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_VERT_BLANKING_STOP));
	ak_sensor_print("tvp5150: Chrominance processing control #1 and #2 = %02x %02x\n",
			tvp5150_sensor_read_register(TVP5150_CHROMA_PROC_CTL_1),
			tvp5150_sensor_read_register(TVP5150_CHROMA_PROC_CTL_2));
	ak_sensor_print("tvp5150: Interrupt reset register B = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_INT_RESET_REG_B));
	ak_sensor_print("tvp5150: Interrupt enable register B = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_INT_ENABLE_REG_B));
	ak_sensor_print("tvp5150: Interrupt configuration register B = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_INTT_CONFIG_REG_B));
	ak_sensor_print("tvp5150: Video standard = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_VIDEO_STD));
	ak_sensor_print("tvp5150: Chroma gain factor: Cb=0x%02x Cr=0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_CB_GAIN_FACT),
			tvp5150_sensor_read_register(TVP5150_CR_GAIN_FACTOR));
	ak_sensor_print("tvp5150: Macrovision on counter = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_MACROVISION_ON_CTR));
	ak_sensor_print("tvp5150: Macrovision off counter = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_MACROVISION_OFF_CTR));
	ak_sensor_print("tvp5150: ITU-R BT.656.%d timing(TVP5150AM1 only)\n",
			(tvp5150_sensor_read_register(TVP5150_REV_SELECT) & 1) ? 3 : 4);
	ak_sensor_print("tvp5150: Device ID = %02x%02x\n",
			tvp5150_sensor_read_register(TVP5150_MSB_DEV_ID),
			tvp5150_sensor_read_register(TVP5150_LSB_DEV_ID));
	ak_sensor_print("tvp5150: ROM version = (hex) %02x.%02x\n",
			tvp5150_sensor_read_register(TVP5150_ROM_MAJOR_VER),
			tvp5150_sensor_read_register(TVP5150_ROM_MINOR_VER));
	ak_sensor_print("tvp5150: Vertical line count = 0x%02x%02x\n",
			tvp5150_sensor_read_register(TVP5150_VERT_LN_COUNT_MSB),
			tvp5150_sensor_read_register(TVP5150_VERT_LN_COUNT_LSB));
	ak_sensor_print("tvp5150: Interrupt status register B = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_INT_STATUS_REG_B));
	ak_sensor_print("tvp5150: Interrupt active register B = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_INT_ACTIVE_REG_B));
	ak_sensor_print("tvp5150: Status regs #1 to #5 = %02x %02x %02x %02x %02x\n",
			tvp5150_sensor_read_register(TVP5150_STATUS_REG_1),
			tvp5150_sensor_read_register(TVP5150_STATUS_REG_2),
			tvp5150_sensor_read_register(TVP5150_STATUS_REG_3),
			tvp5150_sensor_read_register(TVP5150_STATUS_REG_4),
			tvp5150_sensor_read_register(TVP5150_STATUS_REG_5));

	dump_reg_range("Teletext filter 1",   TVP5150_TELETEXT_FIL1_INI,
			TVP5150_TELETEXT_FIL1_END, 8);
	dump_reg_range("Teletext filter 2",   TVP5150_TELETEXT_FIL2_INI,
			TVP5150_TELETEXT_FIL2_END, 8);

	ak_sensor_print("tvp5150: Teletext filter enable = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_TELETEXT_FIL_ENA));
	ak_sensor_print("tvp5150: Interrupt status register A = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_INT_STATUS_REG_A));
	ak_sensor_print("tvp5150: Interrupt enable register A = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_INT_ENABLE_REG_A));
	ak_sensor_print("tvp5150: Interrupt configuration = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_INT_CONF));
	ak_sensor_print("tvp5150: VDP status register = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_VDP_STATUS_REG));
	ak_sensor_print("tvp5150: FIFO word count = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_FIFO_WORD_COUNT));
	ak_sensor_print("tvp5150: FIFO interrupt threshold = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_FIFO_INT_THRESHOLD));
	ak_sensor_print("tvp5150: FIFO reset = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_FIFO_RESET));
	ak_sensor_print("tvp5150: Line number interrupt = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_LINE_NUMBER_INT));
	ak_sensor_print("tvp5150: Pixel alignment register = 0x%02x%02x\n",
			tvp5150_sensor_read_register(TVP5150_PIX_ALIGN_REG_HIGH),
			tvp5150_sensor_read_register(TVP5150_PIX_ALIGN_REG_LOW));
	ak_sensor_print("tvp5150: FIFO output control = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_FIFO_OUT_CTRL));
	ak_sensor_print("tvp5150: Full field enable = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_FULL_FIELD_ENA));
	ak_sensor_print("tvp5150: Full field mode register = 0x%02x\n",
			tvp5150_sensor_read_register(TVP5150_FULL_FIELD_MODE_REG));

	dump_reg_range("CC   data",   TVP5150_CC_DATA_INI,
			TVP5150_CC_DATA_END, 8);

	dump_reg_range("WSS  data",   TVP5150_WSS_DATA_INI,
			TVP5150_WSS_DATA_END, 8);

	dump_reg_range("VPS  data",   TVP5150_VPS_DATA_INI,
			TVP5150_VPS_DATA_END, 8);

	dump_reg_range("VITC data",   TVP5150_VITC_DATA_INI,
			TVP5150_VITC_DATA_END, 10);

	dump_reg_range("Line mode",   TVP5150_LINE_MODE_INI,
			TVP5150_LINE_MODE_END, 8);
	return 0;
}
#endif

void tvp5150_test(void)
{
	int reg, value;

	reg		= TVP5150_VD_IN_SRC_SEL_1;
	value	= 0x00;
	tvp5150_sensor_write_register(reg, value);

	reg		= TVP5150_MISC_CTL;
	value	= 0x0d;
	tvp5150_sensor_write_register(reg, value);

	reg		= TVP5150_DATA_RATE_SEL;
	value	= 0x40;
	tvp5150_sensor_write_register(reg, value);

	reg		= TVP5150_CONF_SHARED_PIN;
	value	= 0x00;
	tvp5150_sensor_write_register(reg, value);
}

void tvp5150_change_parm(void)
{
	int value;

	value = tvp5150_sensor_read_register(TVP5150_STATUS_REG_5);
	switch ((value >> 1) & 0x7) {
		case 0:
			tvp5150_res.width = 700;
			tvp5150_res.height = 240;
			g_fps = 60;
			break;
		case 1:
			tvp5150_res.width = 700;
			tvp5150_res.height = 288;
			g_fps = 50;
			ak_sensor_print("%s B,G,H,I,N PAL\n", __func__);
			break;
		case 2:
			tvp5150_res.width = 700;
			tvp5150_res.height = 288;
			g_fps = 50;
			break;
		case 3:
			tvp5150_res.width = 700;
			tvp5150_res.height = 288;
			g_fps = 50;
			break;
		case 4:
			tvp5150_res.width = 700;
			tvp5150_res.height = 240;
			g_fps = 60;
			break;
		default:
			break;
	}
}
