/**
 * @file camera_imx323.c
 * @brief camera driver file
 * Copyright (C) 2017 Anyka (Guangzhou) Microelectronics Technology Co., Ltd
 * @author 
 * @date
 * @version 1.0
 * @ref
 */ 
#include "ak_sensor_common.h"
#include <linux/delay.h>

#define SENSOR_RESET_LEVEL		0
#define SENSOR_I2C_ADDR         0x34
#define SENSOR_ID               0x0323
#define SENSOR_MCLK             36
#define SENSOR_REGADDR_BYTE     2
#define SENSOR_DATA_BYTE        1
#define MAX_FPS					30
#define DETAULT_FPS				25
#define SENSOR_OUTPUT_WIDTH		(1920 + SENSOR_VALID_OFFSET_X)
#define SENSOR_OUTPUT_HEIGHT	(1080 + SENSOR_VALID_OFFSET_Y)
#define SENSOR_VALID_OFFSET_X	144
#define SENSOR_VALID_OFFSET_Y	24
#define SENSOR_BUS_TYPE		BUS_TYPE_RAW
#define SENSOR_IO_INTERFACE     DVP_INTERFACE
#define SENSOR_IO_LEVEL         SENSOR_IO_LEVEL_1V8

#define OV_MAX(a, b)            (((a) < (b) ) ?  (b) : (a))
#define OV_MIN(a, b)            (((a) > (b) ) ?  (b) : (a))
#define OV_CLIP3(low, high, x)  (OV_MAX(OV_MIN((x), high), low))

#define SKIP_NUM 		        2
#define EXP_EFFECT_FRAMES 	    1
#define VSYNC_ACTIVE_MS		    31
#define MIPI_MHZ          200
#define MIPI_LANE		  2

static int imx323_cmos_updata_d_gain(const unsigned int d_gain);
static int imx323_cmos_updata_a_gain(const unsigned int a_gain);
static int imx323_cmos_updata_exp_time(unsigned int exp_time);
static int imx323_cmos_update_a_gain_timer(void);

static int g_fps        = DETAULT_FPS;
static int to_fps       = DETAULT_FPS;
static int to_fps_value = 0;

static int imx323_sensor_read_register(int reg)
{
	struct ak_sensor_i2c_data i2cdata;

	i2cdata.u8DevAddr           = SENSOR_I2C_ADDR;
	i2cdata.u32RegAddr          = reg;
	i2cdata.u32RegAddrByteNum   = SENSOR_REGADDR_BYTE;
	i2cdata.u32Data             = 0;
	i2cdata.u32DataByteNum      = SENSOR_DATA_BYTE;
	//printk(KERN_ERR "imx323_sensor_read_register:reg[0x%x]++++++++++++++++++++++++++++\n", reg);

	return ak_sensor_read_register(&i2cdata);
}

static int imx323_sensor_write_register(int reg, int data)
{
	struct ak_sensor_i2c_data i2cdata;
	//int tmp;
	int ret;
	
	i2cdata.u8DevAddr           = SENSOR_I2C_ADDR;
	i2cdata.u32RegAddr          = reg;
	i2cdata.u32RegAddrByteNum   = SENSOR_REGADDR_BYTE;
	i2cdata.u32Data             = 0;
	i2cdata.u32DataByteNum      = SENSOR_DATA_BYTE;	
	//printk(KERN_ERR "sc2232_sensor_write_register:reg[0x%x] data[0x%x]+++++++++++++++++++++\n", reg, data);

    	//tmp = ak_sensor_read_register(&i2cdata);
	//if (tmp == data)
		//return 0;
	
	i2cdata.u8DevAddr           = SENSOR_I2C_ADDR;
	i2cdata.u32RegAddr          = reg;
	i2cdata.u32RegAddrByteNum   = SENSOR_REGADDR_BYTE;
	//i2cdata.u32Data             = (data&(~(tmp^data)));
	i2cdata.u32Data                     =data;
	i2cdata.u32DataByteNum      = SENSOR_DATA_BYTE;
	ret=ak_sensor_write_register(&i2cdata);
	//printk(KERN_ERR"1111111xxxxx imx323_sensor_write_register:reg[0x%x]=0x%x  tmp=0x%x data=0x%x i2cdata.u32Data=0x%x",reg,ak_sensor_read_register(&i2cdata),tmp,data,i2cdata.u32Data );
	return ak_sensor_write_register(&i2cdata);
}

static int imx323_init(const AK_ISP_SENSOR_INIT_PARA *para)
{
	int i;
	int value;
	AK_ISP_SENSOR_REG_INFO *preg_info;

	preg_info = para->reg_info;
	for (i = 0; i < para->num; i++)
	{
		value=imx323_sensor_read_register(preg_info->reg_addr);
		//printk(KERN_ERR "before w reg:0x%.x, val:0x%.x, to write:0x%.x\n", preg_info->reg_addr, value, preg_info->value);
		imx323_sensor_write_register(preg_info->reg_addr, preg_info->value);
		value=imx323_sensor_read_register(preg_info->reg_addr);
		//printk(KERN_ERR "after w reg:0x%.x, val:0x%.x\n", preg_info->reg_addr, value);
		preg_info++;
	}

	to_fps_value = imx323_sensor_read_register(0x0340) << 8;
	to_fps_value |= imx323_sensor_read_register(0x0341);

	return 0;
}

static int imx323_read_id(void)
{
    return SENSOR_ID;
}

static int imx323_probe_id(void)
{
    u8  value;
    u32 tmp;
    u32 id = 0;

	
    value = imx323_sensor_read_register(0x0008);
	//printk(KERN_ERR"imx323_sensor_read_register(0x0008)=0x%x \n",imx323_sensor_read_register(0x0008));
    tmp = value << 8;

    value = imx323_sensor_read_register(0x0009);
	//printk(KERN_ERR"imx323_sensor_read_register(0x0009)=0x%x \n",imx323_sensor_read_register(0x0009));
    tmp |= value;

	//printk(KERN_ERR "sc2232_sensor_read_register:reg[0x3000] data[0x%x]+++++++++++++++++++++\n", imx323_sensor_read_register(0x3000));

    if (0x0040 == tmp)
    {
        id = 0x0323;
    }
	
	printk(KERN_ERR "%s id:0x%x\n", __func__, id);

    //return SENSOR_ID;
	if (id != SENSOR_ID)
		goto fail;

    return SENSOR_ID;

fail:
	return 0;
}

static int imx323_get_resolution(int *width, int *height)
{
	*width  = SENSOR_OUTPUT_WIDTH;
	*height = SENSOR_OUTPUT_HEIGHT;

	return 0;
}

static int imx323_get_mclk(void)
{
	return SENSOR_MCLK;
}

static int imx323_get_fps(void)
{
	return g_fps;
}

static int imx323_get_valid_coordinate(int *x, int *y)
{
	*x = SENSOR_VALID_OFFSET_X;
	*y = SENSOR_VALID_OFFSET_Y;

	return 0;
}

static enum sensor_bus_type imx323_get_bus_type(void)
{
	return SENSOR_BUS_TYPE;
}

static int imx323_get_parameter(int param, void *value)
{
	int ret = 0;
	enum sensor_get_param name = (enum sensor_get_param)param;

	switch (name) {
		case GET_MIPI_MHZ:
			*((int *)value) = MIPI_MHZ;
			break;
			
		case GET_VSYNC_ACTIVE_MS:
			*((int *)value) = VSYNC_ACTIVE_MS;
			break;

		case GET_CUR_FPS:
			*((int *)value) = g_fps;
			break;

		case GET_MIPI_LANE:
			*((int *)value) = MIPI_LANE;
			break;

		case GET_INTERFACE:
			*((int *)value) = SENSOR_IO_INTERFACE;
			break;

		case GET_SENSOR_IO_LEVEL:
			*((int *)value) = SENSOR_IO_LEVEL;
			break;

		default:
			ret = -1;
			break;
	}

	return ret;
}

static int imx323_set_power_on(const int pwdn_pin, const int reset_pin)
{   
	ak_sensor_set_pin_as_gpio(reset_pin);
	ak_sensor_set_pin_dir(reset_pin, 1);
	ak_sensor_set_pin_level(reset_pin, SENSOR_RESET_LEVEL);
	ak_sensor_mdelay(10);
	ak_sensor_set_pin_level(reset_pin, !SENSOR_RESET_LEVEL);
	ak_sensor_mdelay(20);   //modify reset time, used to solve no camera interrupt problem

	g_fps = DETAULT_FPS;
	
	return 0;
}

static int imx323_set_power_off(const int pwdn_pin, const int reset_pin)
{
	//sccb software standby mode
	ak_sensor_set_pin_dir(reset_pin, 0);

	 return 0;
}

static int imx323_set_fps(int fps)
{
	int ret = 0;
	int tmp = 0;

	/* tmp = 75M / (reg0x320c~d=2400) / fps */
	switch (fps) {
		case 30:
			 tmp = 0x0470;
			 break;
		case 25:
			tmp = 0x0553;
			break;
		case 20:
			tmp = 0x06a8;
			break;
		case 15:
			tmp = 0x08e0;
			break;
		case 12:
			tmp = 0x0b18;
			break;
		case 10:
			tmp = 0x0d45;
			break;
		case 8:
			tmp = 0x1136;
			break;
		case 5:
			tmp = 0x1a8a;
			break;
		default:
			printk(KERN_ERR "%s set fps=%d fail\n", __func__, fps);
			ret = -EINVAL;
			break;
	}

	if (!ret) 
	{
		//g_fps = fps;
		to_fps       = fps;
		to_fps_value = tmp;
	}

	return ret;
}

static int imx323_set_standby_in(const int pwdn_pin, const int reset_pin)
{
	return 0;
}

static int imx323_set_standby_out(const int pwdn_pin, const int reset_pin)
{
	return 0;
}

static AK_ISP_SENSOR_CB imx323_callback = 
{
	.sensor_init_func 				    = imx323_init,
	.sensor_read_reg_func			    = imx323_sensor_read_register,
	.sensor_write_reg_func			    = imx323_sensor_write_register,
	.sensor_read_id_func			    = imx323_read_id,
	.sensor_update_a_gain_func		    = imx323_cmos_updata_a_gain,
	.sensor_update_d_gain_func		    = imx323_cmos_updata_d_gain,
	.sensor_updata_exp_time_func	    = imx323_cmos_updata_exp_time,
	.sensor_timer_func				    = imx323_cmos_update_a_gain_timer,

	.sensor_probe_id_func			    = imx323_probe_id,	//use IIC bus
	.sensor_get_resolution_func	        = imx323_get_resolution,
	.sensor_get_mclk_func			    = imx323_get_mclk,
	.sensor_get_fps_func			    = imx323_get_fps,
	.sensor_get_valid_coordinate_func	= imx323_get_valid_coordinate,
	.sensor_get_bus_type_func		    = imx323_get_bus_type,
	.sensor_get_parameter_func		    = imx323_get_parameter,

	.sensor_set_power_on_func		    = imx323_set_power_on,
	.sensor_set_power_off_func		    = imx323_set_power_off,
	.sensor_set_fps_func			    = imx323_set_fps,
	.sensor_set_standby_in_func		    = imx323_set_standby_in,
	.sensor_set_standby_out_func	    = imx323_set_standby_out
};

AK_SENSOR_MODULE(imx323_callback, imx323)


static int imx323_cmos_updata_d_gain(const unsigned int d_gain)
{
	return 0;
}

static void imx323_set_fps_async(void)
{
	if (to_fps != g_fps) {
		imx323_sensor_write_register(0x0340, to_fps_value>>8);
		imx323_sensor_write_register(0x0341, to_fps_value & 0xff);

		g_fps = to_fps;
	}
}

static int gain_table[151]=	// from hisi
{
	// INDEX	again 0~71;		dgain >71
	1024,1059,1097,1135,1175,1217,1259,1304,1349,1397,1446,1497,1549,1604,1660,1719,1779,1842,1906, //5.4dB
	1973,2043,2114,2189,2266,2345,2428,2513,2601,2693,2788,2886,2987,3092,3201,3313,3430,3550, 3675, //11.1dB
	3804,3938,4076,4219,4368,4521,4680,4845,5015,5191,5374,5562,5758,5960,6170,6387,6611,6843, 7084, //16.8dB
	7333,7591,7857,8133,8419,8715,9021,9338,9667,10006,10358,10722,11099,11489,11893,12311,12743, //21.9dB
	13191,13655,14135,14631,15146,15678,16229,16799,17390,18001,18633,19288,19966,20668,21394,22146, //26.7dB
	22924,23730,24564,25427,26320,27245,28203,29194,30220,31282,32381,33519,34697,35917,37179,38485, //31.5dB
	39838,41238,42687,44187,45740,47347,49011,50734,52517,54362,56273,58250,60297,62416,64610,66880, //36.3dB
	69230,71663,74182,76789,79487,82281,85172,88165,91264,94471,97791,101228,104785,108468,112279,  //40.8dB
	116225,120310,124537,128914,133444,138134,142988,148013,153215,158599,164172,169942,175914,182096, //45dB
};

static unsigned int _cmos_gains_convert(unsigned int a_gain, unsigned int d_gain,
		unsigned int *a_gain_out ,unsigned int *d_gain_out)
{
	#define DEAD_INDEX	150
	int tmp = 0;
	int i = 0;

	//ak_sensor_print("1111111111111111111  a_gain =%d\n",a_gain);
	
	/*
	 * gain(time) = 10 ^ (gain(dB) / 20)
	 *
	 * a+d gain max = 10 ^ (45/20) = 177.8279
	 * again max ~= 10 ^ (21 / 20) = 11.22018
	 *
	 * */

	imx323_set_fps_async();

	#if 0
	if (a_gain > 45568)	// 178 * 256
		a_gain = 45568;

	tmp = (a_gain - 256) / 302;
	#else
	/*
	a_gain <<= 2;
	if (a_gain >= gain_table[DEAD_INDEX]) {
		a_gain = gain_table[DEAD_INDEX];
		tmp = gain_table[DEAD_INDEX];
	}

	for (i = 1; i <= DEAD_INDEX; i++) {
		if (a_gain < gain_table[i]) {
			tmp = gain_table[i - 1];
			break;
		}
	}*/
	a_gain <<= 2;
	if (a_gain >= gain_table[DEAD_INDEX]) {
		a_gain = gain_table[DEAD_INDEX];
		tmp = DEAD_INDEX;
	}

	for (i = 1; i <= DEAD_INDEX; i++) {
		if (a_gain < gain_table[i]) {
			tmp = i - 1;
			break;
		}
	}
	#endif
	imx323_sensor_write_register(0x301e, tmp);
	return 0;
}

static int imx323_cmos_updata_a_gain(const unsigned int a_gain)
{
	unsigned int tmp_a_gain;
	unsigned int tmp_d_gain;
	unsigned int tmp_a_gain_out;
	unsigned int tmp_d_gain_out;

	tmp_a_gain = a_gain ; //<<2;
	tmp_d_gain = 0;

	return _cmos_gains_convert(tmp_a_gain, tmp_d_gain,  &tmp_a_gain_out ,&tmp_d_gain_out);
}

static int imx323_cmos_updata_exp_time(unsigned int exp_time)
{
	unsigned int exposure_time_ext;
	unsigned int exposure_time_msb;

	//printk("11111111111111111111113333333333  exp_time=0x%x \n",exp_time);
	imx323_set_fps_async();
	
	if((to_fps_value-1) < exp_time)
	{
		exp_time = to_fps_value-1;
	}
	
	exp_time = to_fps_value -1 -exp_time;
	
	exposure_time_ext =(exp_time>>8)&0xf;
	exposure_time_msb =(exp_time)&0xff;
	imx323_sensor_write_register(0x0202,exposure_time_ext);
	imx323_sensor_write_register(0x0203,exposure_time_msb);
	return EXP_EFFECT_FRAMES;
}

static int imx323_cmos_update_a_gain_timer(void)
{
    return 0;
}
