/**
 * @file      sdfilter.h
 * @brief    Anyka Sound Device Module interfaces header file.
 *
 * This file declare Anyka Sound Device Module interfaces.\n
 * Copyright (C) 2008 Anyka (GuangZhou) Software Technology Co., Ltd.
 * @author    Deng Zhou
 * @date    2008-04-10
 * @version V0.0.1
 * @ref
 */

#ifndef __SOUND_FILTER_H__
#define __SOUND_FILTER_H__

#include "medialib_global.h"

#ifdef __cplusplus
extern "C" {
#endif


/** @defgroup Audio Filter library
 * @ingroup ENG
 */
/*@{*/

/* @{@name Define audio version*/
/** Use this to define version string */   
/* ע⣺ṹ޸ģ޸а汾 */
#define AUDIO_FILTER_VERSION_STRING        (T_U8 *)"AudioFilter Version V1.11.01_svn5504_kernal_EQ-aslc"
/** @} */


#define    _SD_VOLCTL_VOLDB_Q  10
#define    _SD_EQ_MAX_BANDS    10
#define    _SD_MDRC_MAX_BANDS  4
#define    _SD_REECHO_DECAY_Q  10

typedef enum
{
	_SD_FILTER_UNKNOWN ,
	_SD_FILTER_EQ ,
	_SD_FILTER_WSOLA ,
	_SD_FILTER_RESAMPLE,
	_SD_FILTER_3DSOUND,
	_SD_FILTER_DENOICE,
	_SD_FILTER_AGC,
	_SD_FILTER_VOICECHANGE,
    _SD_FILTER_PCMMIXER,
    _SD_FILTER_3DENHANCE,
    _SD_FILTER_MVBASS,
    _SD_FILTER_ASLC,
    _SD_FILTER_TONE_DETECTION,
    _SD_FILTER_VOLUME_CONTROL,
    _SD_FILTER_REECHO,
    _SD_FILTER_MDRC,
}T_AUDIO_FILTER_TYPE;

typedef enum
{
    _SD_EQ_MODE_NORMAL,
    _SD_EQ_MODE_CLASSIC,
    _SD_EQ_MODE_JAZZ,
    _SD_EQ_MODE_POP,
    _SD_EQ_MODE_ROCK,
    _SD_EQ_MODE_EXBASS,
    _SD_EQ_MODE_SOFT,
    _SD_EQ_USER_DEFINE,
} T_EQ_MODE;

//to define the filter type
typedef enum
{
    FILTER_TYPE_NO ,
    FILTER_TYPE_HPF ,
    FILTER_TYPE_LPF ,
    FILTER_TYPE_HSF ,
    FILTER_TYPE_LSF ,
    FILTER_TYPE_PF1    //PeaKing filter
}T_EQ_FILTER_TYPE;


typedef enum
{
    _SD_WSOLA_0_5 ,
    _SD_WSOLA_0_6 ,
    _SD_WSOLA_0_7 ,
    _SD_WSOLA_0_8 ,
    _SD_WSOLA_0_9 ,
    _SD_WSOLA_1_0 ,
    _SD_WSOLA_1_1 ,
    _SD_WSOLA_1_2 ,
    _SD_WSOLA_1_3 ,
    _SD_WSOLA_1_4 ,
    _SD_WSOLA_1_5 ,
    _SD_WSOLA_1_6 ,
    _SD_WSOLA_1_7 ,
    _SD_WSOLA_1_8 ,
    _SD_WSOLA_1_9 ,
    _SD_WSOLA_2_0 
}T_WSOLA_TEMPO;

typedef enum
{
    _SD_WSOLA_ARITHMATIC_0 , // 0:WSOLA, fast but tone bab
    _SD_WSOLA_ARITHMATIC_1   // 1:PJWSOLA, slow but tone well
}T_WSOLA_ARITHMATIC;


typedef enum
{
    RESAMPLE_ARITHMETIC_0 = 0,  // 0: ʺã̶ֻܹ֮
    RESAMPLE_ARITHMETIC_1         // 1: ʲ֮
}RESAMPLE_ARITHMETIC;

typedef enum
{
    _SD_OUTSR_UNKNOW = 0,
    _SD_OUTSR_48KHZ = 1,
    _SD_OUTSR_44KHZ,
    _SD_OUTSR_32KHZ,
    _SD_OUTSR_24KHZ,
    _SD_OUTSR_22KHZ,
    _SD_OUTSR_16KHZ,
    _SD_OUTSR_12KHZ,
    _SD_OUTSR_11KHZ,
    _SD_OUTSR_8KHZ
}T_RES_OUTSR;

typedef enum
{
    PITCH_NORMAL = 0,
    PITCH_CHILD_VOICE ,
    PITCH_MACHINE_VOICE,
    PITCH_ECHO_EFFECT,
    PITCH_ROBOT_VOICE,
    PITCH_RESERVE
}T_PITCH_MODES;

typedef enum
{
    VOLCTL_VOL_MUTIPLE = 0,
    VOLCTL_VOL_DB = 2,
}VOLCTL_VOL_MODE;

typedef struct
{
    int num;
    struct 
    {
        int x;
        int y;
    }stone[10];
    int lookAheadTime;  //ms
    int gainAttackTime;  //ms
    int gainReleaseTime;  //ms
}T_FILTER_MILESTONE;

typedef struct
{
    T_U8  bands;
    /* 
     whether need to bypass total limiter
     0: do total limit; 
     1: bypass total limit
    */
    T_U8  limiterBypass;
    /* 
     whether does bandi Chorus to output  
     0: bandi chorus
     1: bandi doff 
    */
    T_U8  bandiDoff[_SD_MDRC_MAX_BANDS];
    /* 
     whether does bandi's DRC bypass
     0: do bandi's DRC; 
     1: bypass bandi's DRC
    */
    T_U8  bandiDrcBypass[_SD_MDRC_MAX_BANDS];

    /* set boundary freqs */
    T_U32 boundaryFreqs[_SD_MDRC_MAX_BANDS-1];

    // define bands' drc para
    int bandsLookAheadTime;  //ms
    struct
    {
        struct 
        {
            int x;
            int y;
        }stone[2];
        int gainAttackTime;  //ms
        int gainReleaseTime;  //ms
    }drcband[_SD_MDRC_MAX_BANDS];

    // define output limiter para
    struct
    {
        struct 
        {
            int x;
            int y;
        }stone[2];
        int lookAheadTime;  //ms
        int gainAttackTime;  //ms
        int gainReleaseTime;  //ms
    }drctotal;
}T_FILTER_MDRC_PARA;


typedef struct
{
    MEDIALIB_CALLBACK_FUN_MALLOC            Malloc;
    MEDIALIB_CALLBACK_FUN_FREE                Free;
    MEDIALIB_CALLBACK_FUN_PRINTF            printf;
    MEDIALIB_CALLBACK_FUN_RTC_DELAY            delay;
    MEDIALIB_CALLBACK_FUN_INVALID_DCACHE    invDcache;
}T_AUDIO_FILTER_CB_FUNS;


typedef struct
{
    T_U32    m_Type;                //media type
    T_U32    m_SampleRate;        //sample rate, sample per second
    T_U16    m_Channels;            //channel number
    T_U16    m_BitsPerSample;    //bits per sample 

    union {
        struct {
            T_EQ_MODE eqmode;
            /* 
            ֵ(db)ע⣺preGain ֵʽΪ (T_S16)(x.xxx*(1<<10))
            */
            T_S16 preGain;      //-12 <= x.xxx <= 12
            
            // For User Presets
            T_U32 bands;      //1~10
            T_U32 bandfreqs[_SD_EQ_MAX_BANDS];
            /* 
            ÿƵֵע⣺bandgains ֵʽΪ (T_S16)(x.xxx*(1<<10))
            */
            T_S16 bandgains[_SD_EQ_MAX_BANDS];  //-12 <= x.xxx <= 12
            /* 
            ÿƵQֵע⣺
            1. bandQֵʽΪ (T_U16)(x.xxx*(1<<10))
            2. bandQΪ0ÿڲĬֵΪ (T_U16)(1.22*(1<<10))
            3. x.xxx < /(2*ƵƵ), x.xxxֵС64.000
            */
            T_U16 bandQ[_SD_EQ_MAX_BANDS];     // q < sr/(2*f)
            T_U16 bandTypes[_SD_EQ_MAX_BANDS];        

            /*
            ڵá_SD_Filter_SetParam()ıEQʱģʽƽɲ
            */
            T_U8   smoothEna;   // 0-ƽ 1-ƽ
            T_U16 smoothTime;  //ƽʱ(ms)0ÿڲĬֵ(256.0*1000/)

            /*** for ffeq dc_remove ***/
            T_U8     dcRmEna;
            T_U32    dcfb;

            /*** for EQ aslc ***/
            T_U8   aslcEna;
            T_U16  aslcLevelMax;
        } m_eq;
        struct {
            T_WSOLA_TEMPO tempo;
            T_WSOLA_ARITHMATIC arithmeticChoice;
        } m_wsola;
        struct{
            T_U8 is3DSurround;
        }m_3dsound;
        struct {
            //Ŀ 1:48k 2:44k 3:32k 4:24K 5:22K 6:16K 7:12K 8:11K 9:8K
            T_RES_OUTSR  outSrindex;

            //볤(bytes)openʱҪ̬ݡ
            //زʱ볤Ȳֵܳ
            T_U32 maxinputlen; 

            // outSrindexֻenumеļϣĿenumֵ֮ʱ
            // ǲʵˣֱĿʵֵ8000 16000 ...
            // ЧoutSrindex=0
            T_U32 outSrFree; 
            
            T_U32 reSampleArithmetic;
            T_U32 outChannel;
        }m_resample;
        struct{
            T_U16 AGClevel;  // make sure AGClevel < 32767
            /* used in AGC_1 */
            T_U32  max_noise;
            T_U32  min_noise;
            /* used in AGC_2 */
            T_U8  noiseReduceDis;  // ǷԴĽ빦
            T_U8  agcDis;  // ǷԴAGC
            /*
            agcPostEnaagcDis==0£ǷAGC2AGC
            0ʾڿagcfilter_controlѾagcģ
            1: ʾֻҪagcgainֵҪagcagcĵߺ
            */
            T_U8  agcPostEna;  
            T_U16 maxGain;  // Ŵ
            T_U16 minGain;  // СŴ
            T_U32 dc_freq;  // hz
            T_U32 nr_range; // 1~300,ԽͽЧԽ
        }m_agc;
        struct{
            T_U32 ASLC_ena;  // 0:disable aslc;  1:enable aslc
            T_U32 NR_Level;  //  0 ~ 4 Խ,Խ
        }m_NR;
        struct{
            T_PITCH_MODES pitchMode;  // 
            /*
             ֻ PITCH_CHILD_VOICE==pitchMode ʱpitchTempoЧ
             pitchTempo ķΧ[0-10], 0~555~10
            */
            T_U8      pitchTempo; 
        }m_pitch;
        struct{
            /*ǷʹܻЧ1Ϊʹܣ0Ϊر*/
            T_S32 reechoEna;  
            /*
            ˥ӣʽΪ (T_S32)(0.xx * (1<<_SD_REECHO_DECAY_Q))
            ҪòΪ0.32 ֵ (T_S32)(0.32 * (1<<_SD_REECHO_DECAY_Q))
            */
            T_S32 degree;      //0-޻Ч
            /*
            ÷С0-300 
            */
            T_U16 roomsize;    //0-Ĭֵ(71)
            /*
            ʱ(ms)೤ʱʧ
            ע⣺õԽҪĻԽԲ̫
                  һ㽨1000ڣڴ㹻ҲôЩ
                  ڴ治Сֵ
            */
            T_U16 reechoTime;  //0-Ĭֵ(840)
            /*
            ǷҪԭʼͬʱ 
            0: ԭʼĶǷ֮
            1ͷ֮һ
            */
            T_U8  needMainBody; //0 or 1
        }m_reecho;
        struct{
            /* 
            ֵ(db)
            ע⣺preGain ֵʽΪ (T_S16)(x.xxx*(1<<10)) 
             -12 <= x.xxx <= 12
            */
            T_S16 preGain;  
            T_S16 cutOffFreq;
            /* 
            3Dȣ
            ע: depthֵʽΪ (T_S16)(x.xxx*(1<<10)), 
             -1 < x.xxx < 1
            */
            T_S16 depth;   
            /*** for 3D Enhance's aslc, resvered***/
            T_U8   aslcEna;
            T_U16  aslcLevelMax;
        }m_3DEnhance;
        struct{
            /* 
            ֵ(db)
            ע⣺preGain ֵʽΪ (T_S16)(x.xxx*(1<<10)) 
             -12 <= x.xxx <= 12
            */
            T_S16 preGain;
            T_S16 cutOffFreq;  
            /* 
            ǿȣ
            ע: bassGain ֵʽΪ (T_S16)(x.xxx*(1<<10)), 
             0 < x.xxx < 12
            */
            T_S16 bassGain;
            /*** for MVBass's aslc ***/
            T_U8   aslcEna;
            T_U16  aslcLevelMax;
        }m_mvBass;
        struct{
            T_BOOL aslcEna;
            T_U16  aslcLimitLevel;  //޷ֵ
            T_U16  aslcStartLevel;  //Ҫ޷ʼ
            /* 
            jointChannels:
               0: 㣬洦
               1: ϵȡֵϲȻһ棬ͬݣ
               2: ݽѰֵ㣬Ȼһֱֵ洦
            */            
            T_U16  jointChannels;

            /*
             maxLeninpcmݳȣֽΪλ
             ΪòʱԤȰ󳤶ȷڴ档
             ݳȷıʱηڴ棬ڴʧܵ
            */
            T_U16  maxLenin;  
        }m_aslc;
        struct{
            /* 
            set volume mode::
            VOLCTL_VOL_MUTIPLE: ֵ volume ֵⲿֵ
            VOLCTL_VOL_DB:      ֵ voldb ֵ ⲿdbֵ
            */
            T_U16 setVolMode;   //0:vol multiple;  2:vol db

            /* 
            ֵ, (T_U16)(x.xx*(1<<10)), x.xx=[0.00~7.99]ʾ
            õֵҪ1.00*(1<<10)Ϊܻᵼʧ
            */
            T_U16 volume; 

            /* 
            DB, ֵʽΪ(T_S32)(x.xx*(1<<10)), x.xx=[-60.00~8.00]
            õֵҪ0dbΪܻᵼʧ
             x.xxx<=-79db,  x.xxx>8.0, ܻᵼ
            */
            T_S32 voldb;

            /* Ϊ˷ֹ任̲pipaƽƽĹʱ */
            T_U16 volSmoothTime;  //ms
        }m_volumeControl;
        struct{
            T_U32 baseFreq;
        }m_toneDetection;
        struct{
            /*
             maxLeninpcmݳȣֽΪλ
             ΪòʱԤȰ󳤶ȷڴ档
             ݳȷıʱηڴ棬ڴʧܵ
            */
            T_U16  maxLenin;  
        }m_mdrc;
    }m_Private;
}T_AUDIO_FILTER_IN_INFO;

typedef struct
{
    T_AUDIO_FILTER_CB_FUNS    cb_fun;
    T_AUDIO_FILTER_IN_INFO    m_info;
    T_AUDIO_CHIP_ID         chip;
    const char    *strVersion;

    const T_VOID            *ploginInfo;
}T_AUDIO_FILTER_INPUT;

typedef struct
{
    T_VOID *buf_in;
    T_U32 len_in;
    T_VOID *buf_out;
    T_U32 len_out;
    T_VOID *buf_in2;  //for mix pcm samples
    T_U32 len_in2;
}T_AUDIO_FILTER_BUF_STRC;

typedef struct
{
    T_AUDIO_FILTER_CB_FUNS cb;
    T_U32    m_Type;
}T_AUDIO_FILTER_LOG_INPUT;

//////////////////////////////////////////////////////////////////////////

/**
 * @brief    ȡЧ汾Ϣ.
 * @author    Deng Zhou
 * @date    2009-04-21
 * @param    [in] T_VOID
 * @return    T_S8 *
 * @retval    Ч汾
 */
T_S8 *_SD_GetAudioFilterVersionInfo(void);

/**
 * @brief    ȡЧ汾Ϣ, ֧Щ.
 * @author  Tang Xuechai
 * @date    2014-05-05
 * @param    [in] T_AUDIO_FILTER_CB_FUNS
 * @return    T_S8 *
 * @retval    ؿ汾
 */
T_S8 *_SD_GetAudioFilterVersions(T_AUDIO_FILTER_CB_FUNS *cb);

/**
 * @brief    Ч豸.
 * @author    Deng Zhou
 * @date    2008-04-10
 * @param    [in] filter_input:
 * Чṹ
 * @return    T_VOID *
 * @retval    Чڲṹָ룬ձʾʧ
 */
T_VOID *_SD_Filter_Open(T_AUDIO_FILTER_INPUT *filter_input);

/**
 * @brief    Ч.
 * @author    Deng Zhou
 * @date    2008-04-10
 * @param    [in] audio_filter:
 * Чڲ뱣ṹ
 * @param    [in] audio_filter_buf:
 * bufferṹ
 * @return    T_S32
 * @retval    Ч⴦ƵݴСbyteΪλ
 */
T_S32 _SD_Filter_Control(T_VOID *audio_filter, T_AUDIO_FILTER_BUF_STRC *audio_filter_buf);

/**
 * @brief    رЧ豸.
 * @author    Deng Zhou
 * @date    2008-04-10
 * @param    [in] audio_decode:
 * Чڲ뱣ṹ
 * @return    T_S32
 * @retval    AK_TRUE :  رճɹ
 * @retval    AK_FLASE :  ر쳣
 */
T_S32 _SD_Filter_Close(T_VOID *audio_filter);

/**
 * @brief    Ч:ٶ,EQģʽ.
 *          m_SampleRate,m_BitsPerSample,m_Channels1Ϊ0,򲻸ıκЧ,AK_TRUE
 * @author    Wang Bo
 * @date    2008-10-07
 * @param    [in] audio_filter:
 * Чڲ뱣ṹ
 * @param    [in] info:
 * ЧϢṹ
 * @return    T_S32
 * @retval    AK_TRUE :  óɹ
 * @retval    AK_FLASE :  쳣
 */
T_S32 _SD_Filter_SetParam(T_VOID *audio_filter, T_AUDIO_FILTER_IN_INFO *info);

/**
 * @brief    ASLCģ޷.
 * @author    Tang Xuechai
 * @date    2015-04-17
 * @param    [in] audio_filter: Чڲ뱣ṹ
 * @param    [in] fmileStones: ASLC޷߲οƵӿ˵ĵ
 * @return    T_S32
 * @retval    AK_TRUE :  óɹ
 * @retval    AK_FLASE :  쳣
 */
T_S32 _SD_Filter_SetAslcMileStones(T_VOID *audio_filter, T_FILTER_MILESTONE *fmileStones);

/**
 * @brief     ASLCģľֵ֮.
 * @author    Tang Xuechai
 * @date    2018-01-22
 * @param    [in] audio_filter: Чڲ뱣ṹ
 * @param    [in] silenceLevel: ֵpcmֵСֵΪǾ
 * @return    T_S32
 * @retval    AK_TRUE :  óɹ
 * @retval    AK_FLASE :  쳣
 */
T_S32 _SD_Filter_SetAslcSilenceLevel(T_VOID *audio_filter, T_U32 silenceLevel);

/**
 * @brief     ASLCģľ֮ʱֵ.
 * @author    Tang Xuechai
 * @date    2018-01-22
 * @param    [in] audio_filter: Чڲ뱣ṹ
 * @param    [in] silenceTime: ʱֵʱ䳬ֵΪ뾲״̬
 * @return    T_S32
 * @retval    AK_TRUE :  óɹ
 * @retval    AK_FLASE :  쳣
 */
T_S32 _SD_Filter_SetAslcSilenceTime(T_VOID *audio_filter, T_U32 silenceTime);

/**
 * @brief    MDRCģ޷.
 * @author    Tang Xuechai
 * @date    2017-07-21
 * @param    [in] audio_filter: Чڲ뱣ṹ
 * @param    [in] fmdrc: MDRCĲοƵӿ˵ĵ
 * @return    T_S32
 * @retval    AK_TRUE :  óɹ
 * @retval    AK_FLASE :  쳣
 */
T_S32 _SD_Filter_SetMdrcPara(T_VOID *audio_filter, T_FILTER_MDRC_PARA *fmdrc);

/**
 * @brief    ز
 * @author    Tang_Xuechai
 * @date        2013-07-03
 * @param    [in] audio_filter:
 *               Чڲ뱣ṹ
 * @param    [out] dstData 
 *               pcm
 * @param    [in] srcData:
 *               pcm
 * @param    [in] srcLen 
 *               pcmݵbyte
 * @return    T_S32
 * @retval    >=0 :  زpcmݵbyte
 * @retval    <0  :  زʧ
 */
T_S32  _SD_Filter_Audio_Scale(T_VOID *audio_filter, T_S16 dstData[], T_S16 srcData[], T_U32 srcLen);


/**
* @brief    EQƵתΪʱ.
* @author    Tang Xuechai
* @date        2015-03-24
* @param    [in] audio_filter:
*           Чڲṹ_SD_Filter_Openķָ
* @param    [in] info:
*           ЧϢṹ
* @return    T_VOID *
* @retval    EQڲȡʱָ룬ձʾʧ
*/
T_VOID *_SD_Filter_GetEqTimePara(T_VOID *audio_filter, T_AUDIO_FILTER_IN_INFO *info);

/**
* @brief    ѵǰҪʹõEQʱݸEQ.
* @author    Tang Xuechai
* @date        2015-03-24
* @param    [in] audio_filter:
*           Чڲṹ_SD_Filter_Openķָ
* @param    [in] peqTime:
*           ʱָ
* @return    T_S32
* @retval    AK_TRUE :  óɹ
* @retval    AK_FLASE:  쳣
*/
T_S32 _SD_Filter_SetEqTimePara(T_VOID *audio_filter, T_VOID *peqTime);

/**
* @brief    ͷEQʱռõĿռ.
* @author    Tang Xuechai
* @date        2015-03-24
* @param    [in] audio_filter:
*           Чڲṹ_SD_Filter_Openķָ
* @param    [in] peqTime:
*           ʱָ
* @return    T_S32
* @retval    AK_TRUE :  óɹ
* @retval    AK_FLASE:  쳣
*/
T_S32 _SD_Filter_DestoryEqTimePara(T_VOID *audio_filter, T_VOID *peqTime);


/**
 * @brief    ģֵ.
 * @author    Tang Xuechai
 * @date    2015-08-11
 * @param    [in] audio_filter: Чڲ뱣ṹ
 * @param    [in] volume: Ŀֵ
 *    ֵ, (T_U16)(x.xx*(1<<10)), x.xx=[0.00~7.99]ʾ
 *    õֵҪ1.00*(1<<10)Ϊܻᵼʧ
 * @return    T_S32
 * @retval    AK_TRUE :  óɹ
 * @retval    AK_FLASE :  쳣
 */
T_S32 _SD_Filter_SetVolume(T_VOID *audio_filter, T_U16 volume);

/**
 * @brief    ģֵ.
 * @author    Tang Xuechai
 * @date    2015-08-11
 * @param    [in] audio_filter: Чڲ뱣ṹ
 * @param    [in] volume: ĿDBֵ
 *    DB, ֵʽΪ(T_S32)(x.xx*(1<<10)), x.xx=[-100.00~8.00], [-60.00~8.00]֮䲽1dbЧ
 *    õֵҪ0dbΪܻᵼʧ
 *     x.xxx<=-79db,  x.xxx>8.0, ܻᵼ
 * @return    T_S32
 * @retval    AK_TRUE :  óɹ
 * @retval    AK_FLASE :  쳣
 */

T_S32 _SD_Filter_SetVolumeDB(T_VOID *audio_filter, T_S32 volume);

/**
* @brief    Ч:  仯ƽʱ
* @param    [in] audio_filter:  Чڲ뱣ṹ
* @param    [in] stime: Ҫõƽʱ䣬λmsָ0dbҪʱ䡣
* @return      T_S32
* @retval       AK_TRUE :  óɹ
* @retval       AK_FLASE :  쳣
*/
T_S32 _SD_Filter_Volctl_SetSmoothTime(T_VOID *audio_filter, T_U32 stime);

/**
* @brief    ȡЧ:  ȡǰЧֵ
* @param    [in] audio_filter:  Чڲ뱣ṹ
* @return      T_S32
* @retval       >=0 :  ȡֵ
* @retval       <0:     ȡʧ
*/
T_S32 _SD_Filter_Volctl_GetCurVolume(T_VOID *audio_filter);

const T_VOID *_SD_EQ_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_3DEnhance_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_3DSound_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_ASLC_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_mvBass_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_NR_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_AGC_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_VolCtl_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_WSOLA_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_pitch_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_Mixer_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_Reecho_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_Resample_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_toneDetection_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);
const T_VOID *_SD_MDRC_login(T_AUDIO_FILTER_LOG_INPUT *plogInput);

#ifdef __cplusplus
}
#endif

#endif
/* end of sdfilter.h */
/*@}*/
