/*
 * Copyright 2006 Freescale Semiconductor
 * York Sun (yorksun@freescale.com)
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#include <common.h>
#include <command.h>
#include <malloc.h>
#include <asm/arch/factory.h>

unsigned short mac_fact_para_checksum(GS_FACTORYPARA *factPara);//dean
extern char        console_buffer[CONFIG_SYS_CBSIZE];

unsigned short mac_fact_para_checksum(GS_FACTORYPARA *factPara)//dean
{
#pragma GCC diagnostic warning "-Wstrict-aliasing"
#pragma GCC diagnostic push
    /* Not portable or guaranteed to work */
    unsigned char factParaBuf[GS_FACTORY_CFG_SIZE];
    unsigned char *p = (unsigned char *)&factParaBuf[0];
    unsigned short sum = 0;
    unsigned short checksum = 0;
    unsigned int length = 0;
    int i;

    length = ( factPara->length[1] << 16 ) | factPara->length[0];
    if ( length == 0xFFFFFFFF ) {
        return 0;
    }

    memcpy(&factParaBuf, factPara, length);
    ((GS_FACTORYPARA*)&factParaBuf[0])->checksum = 0;

    for (i = 0; i < length; i+=2)
    {
        sum += (p[i] << 8 | p[i+1]);
    }

    checksum = 0xffff - sum + 0x1;

    return (checksum >> 8 | checksum << 8);
#pragma GCC diagnostic pop
}

int do_setmac (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    int rcode = 0;
    char factParaBuf[GS_FACTORY_CFG_SIZE];
    GS_FACTORYPARA *factPara = (GS_FACTORYPARA*)factParaBuf;
    ulong newMacAddr = 0;
    ulong newMacAddrHi = 0;
    int inputLen;

    if (argc > 1) {
        printf ("Usage:\n%s\n", cmdtp->usage);
        return 1;
    }

    if (load_fact_para((GS_FACTORYPARA*)factParaBuf) != 0)
    {
        return 1;
    }

    if (factPara->mac[0] == 0xffff && factPara->mac[1] == 0xffff)
    {
        printf ("No factory config data found in flash.\n");
        return 1;
    }

    if (mac_fact_para_checksum(factPara) != factPara->checksum)
    {
        printf ("No factory config data found in flash.\n");
        return 1;
    }
    do {
        inputLen = readline("Input MAC addr: ");
        if (inputLen == 12) 
            break;
        printf ("Invalid MAC address format.\n");
    } while (1);

    if (inputLen == 12)
    {
        /* simple_strtoul will truncate to 32-bits due to no 64-bit multiply */
        newMacAddr = simple_strtoul(console_buffer, NULL, 16);
        console_buffer[4] = 0;
        newMacAddrHi = simple_strtoul( console_buffer, NULL, 16 );
    }
    else
    {
        printf ("Invalid MAC address format.\n");
        return 1;
    }

    factPara->mac[0] = ((newMacAddrHi >> 8) & 0xFF) | (((newMacAddrHi) & 0xFF) << 8);
    factPara->mac[1] = ((newMacAddr >> 24) & 0xFF) | (((newMacAddr >> 16 ) & 0xFF) << 8);
    factPara->mac[2] = ((newMacAddr >> 8) & 0xFF) | (((newMacAddr) & 0xFF) << 8);
    printf("MAC addr set to: %02X:%02X:%02X:%02X:%02X:%02X\n",
        factPara->mac[0] & 0xFF, (factPara->mac[0] >> 8) & 0xFF,
        factPara->mac[1] & 0xFF, (factPara->mac[1] >> 8) & 0xFF,
        factPara->mac[2] & 0xFF, (factPara->mac[2] >> 8) & 0xFF);

    factPara->checksum = mac_fact_para_checksum(factPara);
    printf("Chksum is 0x%x\n", factPara->checksum);

    rcode = save_fact_para((GS_FACTORYPARA*)factParaBuf);
    if (rcode)
    {
        printf("Write factory data failed\n");
    }

    return rcode;
}

U_BOOT_CMD(
    setmac,   1,  1,   do_setmac,
    "setmac - program MAC addr for devices\n",
    "input xxxxxxxxxxxx - 12 digit of MAC\n"
);


