/*
 * Copyright (C) 2011 OMICRON electronics GmbH
 *
 * based on drivers/mtd/nand/nand_spl_load.c
 *
 * Copyright (C) 2011
 * Heiko Schocher, DENX Software Engineering, hs@denx.de.
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#include <common.h>
#include <spi_flash.h>
#include <spl.h>



extern T_FILE_CONFIG g_file_info;

#ifndef CONFIG_SPINAND_FLASH

int spi_get_partition_table (struct spi_flash *spi)
{

	unsigned int *Partion = NULL;
	int ret = 0;
	unsigned long nr_parts;
	unsigned long parts_offset;
	unsigned long parts_backup_offset;
	T_PARTITION_TABLE_INFO *gPartion_table = NULL;
	int i;
	char cmd_buf[8] = {0,};

	
	if (spi_partition_table_offset_get(spi, &parts_offset)){
		printf("get partition table offset failed!\n");
		return -1;
	}

	Partion = (unsigned int *)malloc(15*sizeof(T_PARTITION_TABLE_INFO));
	if(Partion == NULL){
		printf("malloc partion table buffer failed!\n");
		return -1;
	}

	if (spi_partition_table_content_get(spi, parts_offset, Partion, &nr_parts) == -1){
		printf("get partition table backup content!\n");
		if (spi_partition_table_content_get(spi, parts_offset+spi->erase_size, Partion, &nr_parts) == -1){
			printf("get partition table backup content failed!\n");
			free(Partion);
			return -1;
		}
	}

	gPartion_table = (T_PARTITION_TABLE_INFO *)Partion;
	
	printf("g_nr_parts=0x%lx\n", nr_parts);
	for (i=0; i<nr_parts; i++) {
		memset(cmd_buf,0,sizeof(cmd_buf));
		sprintf(cmd_buf, "%s\0", gPartion_table->partition_info.name);
		printf("##gPartion:name:%s, offset:0x%08x, ksize:0x%08x ##\n",
					cmd_buf, 
					gPartion_table->partition_info.start_pos, 
					gPartion_table->partition_info.ksize); 
		if(memcmp(UBOOT_PART_NAME, gPartion_table->partition_info.name, strlen(UBOOT_PART_NAME)) == 0){

			g_file_info.flash_offset = gPartion_table->partition_info.start_pos;
			g_file_info.data_size = gPartion_table->ex_partition_info.parameter1;
			g_file_info.partition_size = gPartion_table->partition_info.ksize*1024;
			g_file_info.load_add = gPartion_table->ex_partition_info.parameter2;
			break;
		}	
		gPartion_table++;
	}


	gPartion_table = (T_PARTITION_TABLE_INFO *)Partion;

	//if no find UBOOT , to find kernel and load
	if(i == nr_parts){
		printf("find the kernel \n");	
		for (i=0; i<nr_parts; i++) {
			memset(cmd_buf,0,sizeof(cmd_buf));
			sprintf(cmd_buf, "%s\0", gPartion_table->partition_info.name);
			printf("##gPartion:name:%s, offset:0x%08x, ksize:0x%08x ##\n",
						cmd_buf, 
						gPartion_table->partition_info.start_pos, 
						gPartion_table->partition_info.ksize); 
			if(memcmp(KERNEL_PART_NAME, gPartion_table->partition_info.name, strlen(KERNEL_PART_NAME)) == 0){

				g_file_info.flash_offset = gPartion_table->partition_info.start_pos;
				g_file_info.data_size = gPartion_table->ex_partition_info.parameter1;
				g_file_info.partition_size = gPartion_table->partition_info.ksize*1024;
				g_file_info.load_add = gPartion_table->ex_partition_info.parameter2;
				break;
			}	
		}
		gPartion_table++;
	}
	free(Partion);
	if(i == nr_parts)
	{
		printf("error: no find the kernel in partition table\n");
		return -1;
	}


	return 0;
}


/*
 * The main entry for SPI booting. It's necessary that SDRAM is already
 * configured and available since this code loads the main U-Boot image
 * from SPI into SDRAM and starts it from there.
 */
void spl_spi_load_image(void)
{
	struct spi_flash *flash;
	struct image_header *header;

	/*
	 * Load U-Boot image from SPI flash into RAM
	 */

	flash = spi_flash_probe(CONFIG_SPL_SPI_BUS, CONFIG_SPL_SPI_CS,
				CONFIG_SF_DEFAULT_SPEED, SPI_MODE_3);
	if (!flash) {
		puts("SPI probe failed.\n");
		hang();
	}

	//partition table
	if(spi_get_partition_table(flash) == -1)
	{
		puts("spinand_get_partition_table failed.\n");
		hang();
	}

	/* use CONFIG_SYS_TEXT_BASE as temporary storage area */
	header = (struct image_header *)(CONFIG_SYS_TEXT_BASE);

	/* Load u-boot, mkimage header is 64 bytes. */
	spi_flash_read(flash, g_file_info.flash_offset, 0x40,
		       (void *)header);
	spl_parse_image_header(header);

	//
	if(spl_image.os == IH_OS_U_BOOT)//reset init
	{
		spl_image.size = g_file_info.data_size;
		spl_image.entry_point = g_file_info.load_add;
		spl_image.load_addr = g_file_info.load_add;
	}
	
	spi_flash_read(flash, g_file_info.flash_offset,
		       spl_image.size, (void *)spl_image.load_addr);
}

#endif
