/*
 * (C) Copyright 2003
 * Texas Instruments.
 * Kshitij Gupta <kshitij@ti.com>
 * Configuation settings for the TI OMAP Innovator board.
 *
 * (C) Copyright 2004
 * ARM Ltd.
 * Philippe Robin, <philippe.robin@arm.com>
 * Configuration for Versatile PB.
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef __CONFIG_H
#define __CONFIG_H

/*need two boot config*/
#define CONFIG_SPL_BOOT

#define CONFIG_GSC36XX           1
#define CONFIG_SHA256            1

#define CONFIG_SPINAND_FLASH
#define CONFIG_SPINAND_128MB

/* #define DEBUG 1  */ /*FIXME:use for debug.*/
/* #define CONFIG_ETHERNET_MAC_MII 1 */
#define CONFIG_ETHERNET_MAC_RMII 1
/* #define CONFIG_SYS_RX_ETH_BUFFER 256 */

#define CONFIG_ANYKA_WATCHDOG   1
/*
 * High Level Configuration Options
 * (easy to change)
 */
#define CONFIG_ARM926EJS	1	/* This is an arm926ejs CPU core */
#define CONFIG_AK3918		1	/* in Versatile Platform Board	*/
#define CONFIG_ARCH_AK39	1	/* Specifically, a Versatile	*/
#define CONFIG_PLAT_ANYKA 	1
#define CONFIG_USE_IRQ      0
#define CONFIG_BOARD_EARLY_INIT_F

/*use for arch/arm/cpu/arm926ejs/start.S */
/*read uboot from 512 bytes offset pos in spi flash 0~511 is irom param.*/


#ifdef  CONFIG_SPINAND_FLASH
#define CONFIG_SYS_TEXT_BASE   0x80eff800    
#else
#define CONFIG_SYS_TEXT_BASE   0x80dffe00   
#endif

/*
 * Physical Memory Map
 */
#define CONFIG_NR_DRAM_BANKS        1          /* we have 1 bank of DRAM */
#define CONFIG_SYS_SDRAM_BASE       0x80000000
#define PHYS_SDRAM_SIZE             (128*1024*1024) /* 128MB of DRAM */
#define CONFIG_SYS_SDRAM_PROTECT_SIZE       0x01000000 /* 16MB of DRAM */
/*
 * Stack sizes
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE	(128*1024)	/* regular stack */
#ifdef CONFIG_USE_IRQ
#define CONFIG_STACKSIZE_IRQ	(8*1024)	/* IRQ stack */
#define CONFIG_STACKSIZE_FIQ	(4*1024)	/* FIQ stack */
#endif
/*
 * Stack should be on the SRAM because
 * DRAM is not init ,(arch/arm/lib/crt0.S)
 */
#define CONFIG_SYS_INIT_SP_ADDR		(0x80ff0000 - GENERATED_GBL_DATA_SIZE)

/*
 * Size of malloc() pool
 */
#define CONFIG_SYS_MALLOC_LEN	(CONFIG_ENV_SIZE + 4096*1024)


/*
 * input clock of PLL
 */
/* ak39 has 12.0000MHz input clock */
#define CONFIG_SYS_CLK_FREQ	12000000

#define CONFIG_SYS_HZ  		(1000)

/*
 * cdh:add print version info
 */
#define CONFIG_VERSION_VARIABLE 1	

/*
 * Flash config
 */
/* #define CONFIG_SYS_NO_FLASH */

/*
 *-----------------------------------------------------------
 * | 0x0 - 0x1FF 		|(2 pages)   chip rom param  
 * -----------------------------------------------------
 * | 0x200 - 0x1FFFF    |(556 pages) u-boot
 * -----------------------------------------------------// 558 page
 * | 0x22000 - 0x220FF  | (1 pages) partition info
 * -----------------------------------------------------
 * | 0x22100 - 0x221FF  | (1 pages) config info
 * -----------------------------------------------------// 560 page
 * | 0x22200 - 0x2221FF | (8192 pages) kernel
 * -----------------------------------------------------
 * -----------------------------------------------------// 8752 page
 * | 0xxxx00 - 0xxxFF | (m pages) MAC (4KB)16page
 * -----------------------------------------------------
 * -----------------------------------------------------// 8768 page
 * | 0xxxx00 - 0xxxxFF | (n pages) serial number(4KB)16page
 * -----------------------------------------------------//8784 page
 * | 0x20000 - 0x21FFF  | (4 pages) env
 * ----------------------------------------------------- //8800 page
 * -----------------------------------------------------
 * | 0xxxx00 - 0xxxxFF | (n pages) fs 
 * -----------------------------------------------------
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *-----------------------------------------------------------
 * */


/* max number of memory banks */
#define CONFIG_SYS_MAX_FLASH_BANKS	1
#define CONFIG_SYS_FLASH_BASE		0x0

/* 0x0 - 0x2page-1*/
#define CONFIG_CHIPROM_PARAM_OFFSET (CONFIG_SYS_FLASH_BASE)
#define CONFIG_CHIPROM_PARAM_SIZE  	(512)

/* 0x2page - 0x558page-1*/
#define CONFIG_UBOOT_OFFSET  		\
	(CONFIG_CHIPROM_PARAM_OFFSET + CONFIG_CHIPROM_PARAM_SIZE)
#define CONFIG_UBOOT_SIZE  			(204288 - 512) // (183808 - 512) // cdh:(142848 - 512)

/* 0x558page - 0x559page-1 */
#define CONFIG_PART_OFFSET 			\
	(CONFIG_UBOOT_OFFSET + CONFIG_UBOOT_SIZE)
#define CONFIG_PART_SIZE 			(256)

/* 0x559page - 0x560page-1 */
#define CONFIG_CFG_OFFSET 			\
	(CONFIG_PART_OFFSET + CONFIG_PART_SIZE)
#define CONFIG_CFG_SIZE 			(256)

/* 0x560page - 0x8752page -1*/
#define CONFIG_KERNEL_OFFSET 		\
	(CONFIG_CFG_OFFSET + CONFIG_CFG_SIZE)
#define CONFIG_KERNEL_SIZE 			(2*1024*1024)

/* 0x8752page - 8768page-1 */
#define CONFIG_FLASH_MAC_OFFSET  			\
	(CONFIG_KERNEL_OFFSET + CONFIG_KERNEL_SIZE) 
#define CONFIG_FLASH_MAC_SIZE  			(4*1024)

/* 8768page - 8784page-1 */
#define CONFIG_FLASH_SER_OFFSET  			\
	(CONFIG_FLASH_MAC_OFFSET + CONFIG_FLASH_MAC_SIZE) 
#define CONFIG_FLASH_SER_SIZE  			(4*1024)


/* enviroment variable offset and size in spi nor flash. According to customer's requirement,it can be modified.*/
#ifdef CONFIG_SPINAND_FLASH
#define CONFIG_ENV_OFFSET  			0x100000
#define CONFIG_CFG_PART_OFFSET      0x80000
#else
#define CONFIG_ENV_OFFSET  			0x47000
#define CONFIG_CFG_PART_OFFSET      0x40000
#endif
	
#define CONFIG_ENV_SIZE  			(4*1024)

#define CONFIG_CMD_FLASH   1
#define CONFIG_AK39_SPI
#define CONFIG_SPI_XFER_CPU
/* #define CONFIG_SPI_XFER_DMA */
#define CONFIG_SPI_FLASH
/* #define CONFIG_CMD_SPI */
#define CONFIG_CMD_SF
/* #define CONFIG_CMD_READCFG */
#define CONFIG_CMD_OPSPRTIF
#define CONFIG_SPI_FLASH_ANYKA 		1
/* #define CONFIG_SPI_FLASH_BUSWIDTH_4X 		1 
#define CONFIG_CMD_SF_TEST */
#define CONFIG_SYS_MAX_FLASH_SECT 	8192 // 256	
/* #define CONFIG_CMD_NAND */ 
#define CONFIG_SYS_MAX_NAND_DEVICE  1
#define CONFIG_SYS_NAND_BASE      0x20120000 
#define CONFIG_SPIFLASH_USE_FAST_READ 1

#define CONFIG_SPI_FLASH_BAR		1

/* ATAG configuration */
/*#define CONFIG_INITRD_TAG
#define CONFIG_SETUP_MEMORY_TAGS */
#define CONFIG_CMDLINE_TAG
#define CONFIG_CMDLINE_EDITING
#define CONFIG_AUTO_COMPLETE

/* MMC */
/* #define CONFIG_CMD_FAT    1 */
#define CONFIG_PARTITIONS 1
/*#define CONFIG_DOS_PARTITION		1
#define CONFIG_MMC     1
#define HAVE_BLOCK_DEVICE		1
#define CONFIG_CMD_MMC
#define CONFIG_GENERIC_MMC		1
#define CONFIG_ANYKA_SDHSMMC		1  */

/*
 * Miscellaneous configurable options
 */
#define CONFIG_LONGHELP
#define CONFIG_SYS_LONGHELP	1
#define CONFIG_SYS_PROMPT	"anyka # "
#define CONFIG_SYS_CBSIZE	512
#define CONFIG_SYS_PBSIZE	(CONFIG_SYS_CBSIZE+sizeof(CONFIG_SYS_PROMPT)+16)
#define CONFIG_SYS_MAXARGS	32
#define CONFIG_SYS_BARGSIZE	CONFIG_SYS_CBSIZE

/* default load address	*/
#define CONFIG_SYS_LOAD_ADDR		0x82008000//0x81008000

/*
 * Check if key already pressed
 * Don't check if bootdelay < 0
 */
#define CONFIG_ZERO_BOOTDELAY_CHECK
#define CONFIG_BOOTDELAY 	        1
#define CONFIG_BOOTABORTKEY		    0x1b /* ascii for escape */

#define CONFIG_CFG_DEF_ADDR 		0x81000000
#define CONFIG_CFG_FILE_CNT 		0x8  //  define the max cnt, because all totale 256B, and every unit 32B

#define CONFIG_ENV_SPI_BUS	0
#define CONFIG_ENV_SPI_CS	0
#define CONFIG_ENV_SPI_MAX_HZ	10000000
#define CONFIG_ENV_SPI_MODE	SPI_MODE_0

#define CONFIG_SF_DEFAULT_BUS	0
#define CONFIG_SF_DEFAULT_CS	0
#define CONFIG_SF_DEFAULT_SPEED	10000000
#define CONFIG_SF_DEFAULT_MODE	SPI_MODE_0

#define CONFIG_ROOT_NAME 	"A"
#define CONFIG_BIOS_NAME 	"KERNEL"
#define CONFIG_ENV_NAME 	"ENV"

#define CONFIG_SECTION_SIZE 4096    // cdh:add for erase section size
#define CONFIG_PARATION_TAB_SIZE 512    // cdh:add for paration table section size

#define CONFIG_SPIP_START_PAGE 0    // cdh:add for searching SPIP flag start flash offset address
#define CONFIG_SPIP_PAGE_CNT  3     // cdh:add for searching SPIP flag
#define CONFIG_PARATION_PAGE_CNT  2     // cdh:add for searching  paration table page cnt


#define CONFIG_ENV_IS_IN_SPI_FLASH 	1
#define CONFIG_ENV_SECT_SIZE 	CONFIG_ENV_SIZE

#define CONFIG_CMD_MTDPARTS  /* Enable MTD parts commands    */
#define CONFIG_MTD_PARTITIONS
#define CONFIG_MTD_DEVICE
#define CONFIG_CMD_LOAD
#define CONFIG_RBTREE
#define CONFIG_CMD_UBI

/* customer need detecting gpio to meet their self-defined requirement. */
#define CONFIG_AK39_GPIO

/* #define CONFIG_TF_UPDATE */

/* According to customer different requirement,we defined three kind of partition table:
** 1.First one is partition table defined by anyka. Saved in partition env by anyka burntool. 
** 2.Second one is partition table defined by customer. Saved in partition env by nor flash programer,beacause
** customer don't wanna modify uboot code.
** 3.Third one is partition table defined by customer self-defined. Saved in uboot code,so if you wanna change
** partition table, you have to modify uboot code.
** You can only choose one from below three.
*/

/* #define CONFIG_PARTITION_TABLE_BY_ANYKA */
#define CONFIG_PARTITION_TABLE_BY_CUSTOMER_ENV 
/* #define CONFIG_PARTITION_TABLE_BY_SELF_DEFINED */

#ifdef CONFIG_PARTITION_TABLE_BY_ANYKA
#define CONFIG_BOOTARGS "console=ttySAK0,115200n8 root=/dev/mtdblock4 rootfstype=squashfs init=/sbin/init " \
                        "mem=128M " \
                        "memsize=128M"

#endif 

#ifdef CONFIG_PARTITION_TABLE_BY_SELF_DEFINED
#define CONFIG_BOOTARGS "console=ttySAK0,115200n8 root=/dev/mtdblock4 "\
			"rootfstype=squashfs init=/sbin/init mem=128M memsize=128M mtdparts=spi0.0:2048K@0x36000(KERNEL)," \
	"1136K@0x238000(ROOT),500K@0x354000(USR.JFFS2),4200K@0x3d1000(USR.SQUASH),"	\
	"4K@0x236000(MAC),4K@0x237000(ENV)"

#endif

/* Obtain partition table by mtdparts */
#ifdef CONFIG_PARTITION_TABLE_BY_CUSTOMER_ENV
#define CONFIG_BOOTARGS "console=ttySAK0,115200n8 rootdevice=rootfsname=ubi0:base rootfstype=squashfs mem=128M" 
#ifdef CONFIG_SPINAND_FLASH
#define MTDIDS_DEFAULT  "nand0=spi0.0"
#else
#define MTDIDS_DEFAULT  "nor0=spi0.0"
#endif
#endif

#ifdef CONFIG_SPINAND_FLASH
#ifdef CONFIG_SPINAND_128MB
#define MTDPARTS_DEFAULT \
        "mtdparts=spi0.0:" \
        "512k(boot)," \
        "512k(factory)," \
        "512K(env)," \
        "2560k(nvram)," \
        "1024k(uboota)," \
        "1024k(ubootb)," \
        "50M(ubi0)," \
        "50M(ubi1)," \
        "-(data)\0"
#else
#define MTDPARTS_DEFAULT \
        "mtdparts=spi0.0:" \
        "2048k(boot)," \
        "512k(factory)," \
        "512K(env)," \
        "1024k(nvram)," \
        "1024k(uboota)," \
        "1024k(ubootb)," \
        "24M(ubi0)," \
        "24M(ubi1)," \
        "-(data)\0"
#endif
#else
#define MTDPARTS_DEFAULT \
        "mtdparts=spi0.0:" \
        "256k(boot)," \
        "28k(factory)," \
        "4K(env)," \
        "96k(nvram)," \
        "256k(uboota)," \
        "256k(ubootb)," \
        "15424K(ubi0)," \
        "15424k(ubi1)," \
        "-(data)\0"
#endif

#define CONFIG_DEFAULT_MTDPARTS "mtdparts=" MTDPARTS_DEFAULT

#ifdef CONFIG_SPINAND_FLASH

#ifdef CONFIG_SPINAND_128MB
#define CONFIG_EXTRA_ENV_SETTINGS \
	"sf_hz=10000000\0" \
	"loadaddr=0x82208000\0" \
	"loadaddr_hdr=0x82207e00\0" \
	"console=ttySAK0,115200n8\0" \
	"gxp_console=gxp_console=0\0" \
    "rootdevice=rootfsname=ubi0:base\0" \
	"memsize=128M\0" \
	"rootfstype=squashfs\0" \
	"ethaddr=00:0B:82:00:00:00\0" \
	"read_kernel=ubi part ubi0; ubi read ${loadaddr_hdr} core\0" \
    "bootcmd=gsboot; run detected_bootcmd\0" \
	"flashboot=setenv bootargs console=${console} ${gxp_console} gluebi=ubi0:ubi1 ubi.mtd=ubi0 ubi.mtd=ubi1 ${rootdevice} rootfstype=${rootfstype} mem=${memsize} ${mtdparts}; run read_kernel; bootm ${loadaddr}\0" \
	"tftpboot=setenv bootargs console=${console} ${gxp_console} gluebi=ubi0:ubi1 ubi.mtd=ubi0 ubi.mtd=ubi1 ${rootdevice} rootfstype=${rootfstype} mem=${memsize} ${mtdparts}; tftp uImage.gsc36xx; bootm ${loadaddr}\0" \
	"updateall=run updateloader updatefact updateboot updatefirmware clearenv clearnvram cleardata\0" \
    "updatefact=load_cfg; tftp ${MODEL}_${PNSTR}${FACTVER}.cfg; sf erase 0x80000 0x80000; sf write ${loadaddr} 0x80000 ${filesize}\0" \
    "updateloader=load_cfg; tftp ${GSMODEL}loader.bin; sf erase 0 0x80000; sf write ${loadaddr} 0 ${filesize}\0" \
    "updateboot=load_cfg; tftp ${GSMODEL}bootdev.bin; sf erase 0x400000 0x100000; sf write ${loadaddr} 0x400000 ${filesize}; sf erase 0x500000 0x100000; sf write ${loadaddr} 0x500000 ${filesize}\0" \
    "updatefirmware=load_cfg; tftp ${GSMODEL}firmware.bin; sf erase 0x600000 0x3200000; sf write ${loadaddr} 0x600000 ${filesize}; sf erase 0x3800000 0x3200000; sf write ${loadaddr} 0x3800000 ${filesize}\0" \
    "clearenv=sf erase 0x100000 0x80000\0" \
    "clearnvram=sf erase 0x180000 0x280000\0" \
    "cleardata=sf erase 0x6A00000 0x1600000\0" 
#else
#define CONFIG_EXTRA_ENV_SETTINGS \
	"sf_hz=20000000\0" \
	"loadaddr=0x82208000\0" \
	"loadaddr_hdr=0x82207e00\0" \
	"console=ttySAK0,115200n8\0" \
	"gxp_console=gxp_console=0\0" \
    "rootdevice=rootfsname=ubi0:base\0" \
	"memsize=128M\0" \
	"rootfstype=squashfs\0" \
	"ethaddr=00:0B:82:00:00:00\0" \
	"read_kernel=ubi part ubi0; ubi read ${loadaddr_hdr} core\0" \
    "bootcmd=gsboot; run detected_bootcmd\0" \
	"flashboot=setenv bootargs console=${console} ${gxp_console} gluebi=ubi0:ubi1 ubi.mtd=ubi0 ubi.mtd=ubi1 ${rootdevice} rootfstype=${rootfstype} mem=${memsize} ${mtdparts}; run read_kernel; bootm ${loadaddr}\0" \
	"tftpboot=setenv bootargs console=${console} ${gxp_console} gluebi=ubi0:ubi1 ubi.mtd=ubi0 ubi.mtd=ubi1 ${rootdevice} rootfstype=${rootfstype} mem=${memsize} ${mtdparts}; tftp uImage.gsc36xx; bootm ${loadaddr}\0" \
	"updateall=run updateboot updatefact updatefirmware\0" \
    "updatefact=load_cfg; tftp ${MODEL}_${PNSTR}${FACTVER}.cfg; sf erase 0x200000 0x80000; sf write ${loadaddr} 0x200000 ${filesize}\0" \
    "updateloader=load_cfg; tftp ${GSMODEL}loader.bin; sf erase 0 0x80000; sf write ${loadaddr} 0 ${filesize}\0" \
    "updateboot=load_cfg; tftp ${GSMODEL}bootdev.bin; sf erase 0x400000 0x100000; sf write ${loadaddr} 0x400000 ${filesize}; sf erase 0x500000 0x100000; sf write ${loadaddr} 0x500000 ${filesize}\0" \
    "updatefirmware=load_cfg; tftp ${GSMODEL}firmware.bin; sf erase 0x600000 0x1800000; sf write ${loadaddr} 0x600000 ${filesize}; sf erase 0x1e00000 0x1800000; sf write ${loadaddr} 0x1e00000 ${filesize}\0" \
    "clearenv=sf erase 0x280000 0x80000\0" \
    "clearnvram=sf erase 0x300000 0x100000\0" \
    "cleardata=sf erase 0x3600000 0x4A00000\0" 
#endif

#else

#define CONFIG_EXTRA_ENV_SETTINGS \
	"sf_hz=20000000\0" \
	"loadaddr=0x82208000\0" \
	"loadaddr_hdr=0x82207e00\0" \
	"console=ttySAK0,115200n8\0" \
	"gxp_console=gxp_console=0\0" \
    "rootdevice=rootfsname=ubi0:base\0" \
	"memsize=128M\0" \
	"rootfstype=squashfs\0" \
	"ethaddr=00:0B:82:00:00:00\0" \
	"read_kernel=ubi part ubi0; ubi read ${loadaddr_hdr} core\0" \
    "bootcmd=gsboot; run detected_bootcmd\0" \
	"flashboot=setenv bootargs console=${console} ${gxp_console} gluebi=ubi0:ubi1 ubi.mtd=ubi0 ubi.mtd=ubi1 ${rootdevice} rootfstype=${rootfstype} mem=${memsize} ${mtdparts}; run read_kernel; bootm ${loadaddr}\0" \
	"tftpboot=setenv bootargs console=${console} ${gxp_console} gluebi=ubi0:ubi1 ubi.mtd=ubi0 ubi.mtd=ubi1 ${rootdevice} rootfstype=${rootfstype} mem=${memsize} ${mtdparts}; tftp uImage.gsc36xx; bootm ${loadaddr}\0" \
	"updateall=run updateboot updatefact updatefirmware\0" \
    "updatefact=load_cfg; tftp ${MODEL}_${PNSTR}${FACTVER}.cfg; sf erase 40000 7000; sf write ${loadaddr} 40000 ${filesize}\0" \
    "updateboot=load_cfg; tftp ${GSMODEL}bootdev.bin; sf erase 0 40000; sf write ${loadaddr} 0 ${filesize}\0" \
    "updatefirmware=load_cfg; tftp ${GSMODEL}firmware.bin; sf erase E0000 F10000; sf write ${loadaddr} E0000 ${filesize}; sf erase FF0000 F10000; sf write ${loadaddr} FF0000 ${filesize}\0" \
    "clearenv=sf erase 0x47000 0x1000\0" \
    "clearnvram=sf erase 0x48000 0x18000\0" \
    "cleardata=sf erase 1f00000 100000\0" 
#endif

#define CONFIG_CMD_RUN 
#define CONFIG_CMD_MEMORY 
#define CONFIG_CMD_SAVEENV

/*
 * Hardware drivers
 */
#define CONFIG_AK39_SERIAL
#define CONFIG_SERIAL1
#define CONFIG_AK_ETHER
#define CONFIG_CMD_NET 
#define CONFIG_CMD_PING
#define CONFIG_CMD_ECHO

/* Define default IP addresses */
#define CONFIG_IPADDR		192.168.211.50	/* own ip address */
#define CONFIG_SERVERIP		192.168.211.200 /* used for tftp (not nfs?) */
#define CONFIG_NETMASK      255.255.255.0    

/* valid baudrates */
#define CONFIG_SYS_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }
#define CONFIG_BAUDRATE		115200

/*
 * For NOR boot, we must set this to the start of where NOR is mapped
 * in memory.
 */
#ifdef CONFIG_NOR_BOOT
#define CONFIG_SYS_TEXT_BASE		0x08000000

#endif

#ifdef CONFIG_SPL_BOOT
/* defines for SPL */
#define CONFIG_SPL
#define CONFIG_SPL_FRAMEWORK
#define CONFIG_SPL_BOARD_INIT
#define CONFIG_SYS_SPL_MALLOC_START	(CONFIG_SYS_TEXT_BASE - \
						CONFIG_SYS_MALLOC_LEN)
#define CONFIG_SYS_SPL_MALLOC_SIZE	CONFIG_SYS_MALLOC_LEN
#define CONFIG_SPL_SERIAL_SUPPORT
#define CONFIG_SPL_LIBCOMMON_SUPPORT
#define CONFIG_SPL_LIBGENERIC_SUPPORT
/*#define CONFIG_SPL_LDSCRIPT	"board/$(BOARDDIR)/u-boot-spl-ipam390.lds"*/
#define CONFIG_SPL_STACK	0x8001ff00

#ifdef  CONFIG_SPINAND_FLASH
#define CONFIG_SPL_TEXT_BASE   0x80fff800    
#else
#define CONFIG_SPL_TEXT_BASE   0x80effe00   
#endif

#define CONFIG_SPL_MAX_SIZE	        0x20000
#define CONFIG_SPL_MAX_FOOTPRINT	0x20000

/* defines for support spi */
#define CONFIG_SPL_SPI_SUPPORT
#define CONFIG_SPL_SPI_FLASH_SUPPORT
#define CONFIG_SPL_SPI_LOAD

#define CONFIG_SPL_SPI_BUS		0
#define CONFIG_SPL_SPI_CS		0
#define CONFIG_SYS_SPI_U_BOOT_OFFS	0x20000

/* add FALCON boot mode */
#define CONFIG_CMD_SPL
#define CONFIG_SPL_OS_BOOT
#define CONFIG_SYS_NAND_SPL_KERNEL_OFFS	0x00200000
#define CONFIG_SYS_SPL_ARGS_ADDR	0x00180000
#define CONFIG_CMD_SPL_NAND_OFS		0x00180000
#define CONFIG_CMD_SPL_WRITE_SIZE	0x400
#endif

#endif	/* __CONFIG_H */
